from unittest import TestCase
from unittest.mock import MagicMock
from reliabilly.components.monitoring import MonitoringProvider, monitor
from reliabilly.settings import Constants


class MonitoringTests(TestCase):
    @staticmethod
    def test_provider():
        mock = MagicMock()
        mock.service_name = Constants.DUMMY
        provider = MonitoringProvider()
        provider.worker_setup(mock)
        provider.worker_result(mock, True)

    def test_decorator(self):
        mock = MagicMock()
        mock.__name__ = Constants.DUMMY
        self.assertTrue(monitor(mock))
        monitor(mock)(mock)

    def test_monitor_ids(self):
        service = 'test'
        method = 'get'
        monitor_prefix = f'{Constants.MONITOR_PREFIX}{service}.{method}'
        monitor_client = MonitoringProvider()
        self.assertEqual(monitor_client.get_monitor_id(service, method, False, False, False),
                         f'{monitor_prefix}{Constants.MONITOR_CALLS_SUFFIX}')
        self.assertEqual(monitor_client.get_monitor_id(service, method, False, True, False),
                         f'{monitor_prefix}{Constants.MONITOR_LARGE_KEY}{Constants.MONITOR_CALLS_SUFFIX}')
        self.assertEqual(monitor_client.get_monitor_id(service, method, False, True, True),
                         f'{monitor_prefix}{Constants.MONITOR_LARGE_KEY}'
                         f'{Constants.MONITOR_ERROR_KEY}{Constants.MONITOR_CALLS_SUFFIX}')
        self.assertEqual(monitor_client.get_monitor_id(service, method, True, False, False),
                         f'{monitor_prefix}{Constants.MONITOR_ELAPSED_SUFFIX}')
        self.assertEqual(monitor_client.get_monitor_id(service, method, True, True, False),
                         f'{monitor_prefix}{Constants.MONITOR_LARGE_KEY}{Constants.MONITOR_ELAPSED_SUFFIX}')
        self.assertEqual(monitor_client.get_monitor_id(service, method, True, True, True),
                         f'{monitor_prefix}{Constants.MONITOR_LARGE_KEY}'
                         f'{Constants.MONITOR_ERROR_KEY}{Constants.MONITOR_ELAPSED_SUFFIX}')

    def test_utility_methods(self):
        mock = MagicMock()
        monitor_client = MonitoringProvider()
        mock.args = "<Request 'http://localhost:8000/test/?f=j&limit=1000' [GET]>"
        result = monitor_client.check_large_page(mock)
        self.assertTrue(result)
        mock.args = "<Request 'http://localhost:8000/test/' [GET]>"
        result = monitor_client.check_large_page(mock)
        self.assertFalse(result)
        mock.args = "<Request 'http://localhost:8000/test/?f=j&limit=10' [GET]>"
        result = monitor_client.check_large_page(mock)
        self.assertFalse(result)
        mock.args = "<Request 'http://localhost:8000/test/?limit=1000' [GET]>"
        result = monitor_client.check_large_page(mock)
        self.assertTrue(result)
        mock.args = ''
        result = monitor_client.check_large_page(mock)
        self.assertFalse(result)



import json
import pickle
from datetime import datetime
from os import path, environ
from boto3 import client
from invoke import task

PROD_ENV = 'prod'
UAT_ENV = 'uat'
UAT_REGION = 'us-west-2'
PROD_REGION = 'us-east-1'
SESSION_FILE_NAME = '.assumed_session'
DEVELOPER_SVC_ROLE = 'arn:aws:iam::640077214053:role/Developer'
ASSUME_ROLE_CMD = f'aws sts assume-role --role-arn {DEVELOPER_SVC_ROLE} ' \
                  f'--role-session-name debug'
UAT_CLUSTER_ARN = f'arn:aws:ecs:{UAT_REGION}:640077214053:cluster/' \
                   'reliabilly-ecs-cluster-uat'
PROD_CLUSTER_ARN = f'arn:aws:ecs:{PROD_REGION}:640077214053:cluster/' \
                   'reliabilly-ecs-cluster'
AWS_CLI_DESCRIBE_CLUSTER_CMD = \
    'aws ecs list-container-instances --cluster ' \
    '{cluster} --region {region} '
AWS_CLI_DESCRIBE_INSTANCES_CMD = \
    'aws ecs describe-container-instances --cluster ' \
    '{cluster} --region {region} --container-instances {instance_ids}'
AWS_DESCRIBE_INSTANCES_CMD = 'aws ec2 describe-instances ' \
                             '--instance-ids {instance_ids} --region {region}'
AWS_SSM_SEND_CMD = 'aws ssm send-command ' \
                   '--document-name "AWS-RunShellScript" --region {region}' \
                   ' --instance-ids {instance_ids} --parameters ' \
                   'commands="{command}" ' \
                   '--output text --query "Command.CommandId"'
AWS_SSM_RESULTS_CMD = 'aws ssm list-command-invocations --command-id ' \
                      '"{command_id}" --details --region {region}'


def get_instance_ids(ctx, env):
    instances = list()
    cluster, region = get_cluster_details(env)
    result = ctx.run(AWS_CLI_DESCRIBE_CLUSTER_CMD.format(
        cluster=cluster, region=region), hide=True).stdout
    instance_arns = ' '.join(json.loads(result)['containerInstanceArns'])
    instance_command = AWS_CLI_DESCRIBE_INSTANCES_CMD.format(
        cluster=cluster, region=region, instance_ids=instance_arns)
    result = json.loads(ctx.run(instance_command,
                                hide=True).stdout)['containerInstances']
    for instance in result:
        instance_id = instance['ec2InstanceId']
        instances.append(instance_id)
    return instances


def get_cluster_details(env):
    cluster = UAT_CLUSTER_ARN
    region = UAT_REGION
    if env != UAT_ENV:
        cluster = PROD_CLUSTER_ARN
        region = PROD_REGION
    return cluster, region


def get_instance_ips(ctx, env):
    ip_addresses = dict()
    _, region = get_cluster_details(env)
    instances = ' '.join(get_instance_ids(ctx, env))
    instance_command = AWS_DESCRIBE_INSTANCES_CMD.format(
        instance_ids=instances, region=region)
    result = json.loads(ctx.run(
        instance_command,
        hide=True).stdout)['Reservations']
    for instance in result:
        instance_id = instance['Instances'][0]['InstanceId']
        ip_address = instance['Instances'][0]['PrivateIpAddress']
        ip_addresses[instance_id] = ip_address
    return ip_addresses


def run_ssm_command(ctx, command, env):
    instances = ' '.join(get_instance_ids(ctx, env))
    _, region = get_cluster_details(env)
    command = AWS_SSM_SEND_CMD.format(
        instance_ids=instances, command=command, region=region)
    command_id = ctx.run(command, hide=True).stdout.strip()
    print(f'Command ID: {command_id}')
    result = json.loads(ctx.run(
        AWS_SSM_RESULTS_CMD.format(command_id=command_id, region=region),
        hide=True).stdout)['CommandInvocations']
    for instance in result:
        print(f'Instance ID: {instance["InstanceId"]}')
        print(instance['CommandPlugins'][0]['Output'])


def get_env(prod=False):
    if prod:
        return PROD_ENV
    return UAT_ENV


def get_mfa_serial():
    config_path = path.expanduser('~') + '/.aws/config'
    with open(config_path, 'r') as file:
        for line in file:
            if line.startswith('mfa_serial'):
                return line.split('=')[1].strip()
    return None


def get_assumed_credentials(token, duration=3600):
    mfa_serial = get_mfa_serial()
    response = client('sts').assume_role(
        DurationSeconds=duration,
        RoleArn=DEVELOPER_SVC_ROLE,
        RoleSessionName='debug', SerialNumber=mfa_serial, TokenCode=token)
    return response


def get_existing_session(token, duration):
    session_file_name = SESSION_FILE_NAME
    if path.exists(session_file_name):
        with open(session_file_name, 'rb') as session_file:
            session = pickle.load(session_file)
            expiration = session['Credentials']['Expiration']
            if datetime.now(expiration.tzinfo) < expiration:
                return session
    if not token:
        print('You need to send your MFA token with the assume call.')
        exit(1)
    return get_assumed_credentials(token, duration)


def update_existing_session(session):
    with open(SESSION_FILE_NAME, 'wb') as session_file:
        pickle.dump(session, session_file, pickle.HIGHEST_PROTOCOL)


def assume(token=None, duration=3600):
    response = get_existing_session(token, duration)
    aws_key = response['Credentials']['AccessKeyId']
    aws_secret = response['Credentials']['SecretAccessKey']
    aws_session = response['Credentials']['SessionToken']
    environ['AWS_ACCESS_KEY_ID'] = aws_key
    environ['AWS_SECRET_ACCESS_KEY'] = aws_secret
    environ['AWS_SESSION_TOKEN'] = aws_session
    environ['AWS_SECURITY_TOKEN'] = aws_session
    update_existing_session(response)


@task
def list_instance_ids(ctx, prod=False):
    assume()
    print(get_instance_ids(ctx, get_env(prod)))


@task
def list_ips(ctx, prod=False):
    assume()
    print(get_instance_ips(ctx, get_env(prod)))


@task
def list_container(ctx, service, all_containers=False, prod=False):
    assume()
    if all_containers:
        run_ssm_command(ctx, 'docker ps -a | grep {0}'.format(
            service), get_env(prod))
        return
    run_ssm_command(ctx, 'docker ps | grep {0}'.format(
        service), get_env(prod))


@task
def list_containers(ctx, all_containers=False, prod=False):
    assume()
    if all_containers:
        run_ssm_command(ctx, 'docker ps -a', get_env(prod))
        return
    run_ssm_command(ctx, 'docker ps', get_env(prod))


@task
def list_images(ctx, prod=False):
    assume()
    run_ssm_command(ctx, 'docker images', get_env(prod))


@task
def logs(ctx, service, prod=False):
    assume()
    run_ssm_command(ctx, 'docker logs --tail 35 {0}'.format(
        service), get_env(prod))


@task
def disk(ctx, prod=False):
    assume()
    run_ssm_command(ctx, 'df -H', get_env(prod))


@task
def uptime(ctx, prod=False):
    assume()
    run_ssm_command(ctx, 'uptime', get_env(prod))



import re

from github import Github, GithubException
from reliabilly.settings import Constants


def get_default_svn_client(token, baseurl=Constants.SOURCE_CONTROL_URL_DEFAULT):
    client = None
    try:
        # This accepts anything as creds and only throws errors on being used
        # A 404 would be valid while a 401 would not be?
        client = Github(base_url=baseurl, login_or_token=token)
    except GithubException as ex:
        print(ex)
        exit(False)

    return client


class SourceControlClient:
    def __init__(self, token, url=Constants.SOURCE_CONTROL_URL_DEFAULT, client=None):
        self.client = client
        if not self.client:
            self.client = get_default_svn_client(token, url)

    def get_organisation(self, org_name):
        try:
            return self.client.get_organization(org_name)
        except GithubException:
            return False

    def get_user(self, username):
        try:
            return self.client.get_user(username)
        except GithubException:
            return False

    def get_user_repository(self, username, repository_name):
        try:
            user = self.get_user(username)
            return user.get_repo(repository_name)
        except GithubException:
            return False

    def get_organisation_repository(self, org_name, repo_name):
        try:
            organisation = self.get_organisation(org_name)
            return organisation.get_repo(repo_name)
        except GithubException:
            return False

    def get_organisation_commit(self, org_name, repo_name, commit_sha):
        try:
            organisation = self.get_organisation(org_name)
            repo = organisation.get_repo(repo_name)
            return repo.get_commit(commit_sha)
        except GithubException:
            return False

    def get_organisation_pull_request(self, org_name, repo_name, number):
        try:
            organisation = self.get_organisation(org_name)
            repo = organisation.get_repo(repo_name)
            return repo.get_pull(int(number))
        except GithubException:
            return False

    def get_pr_for_commit(self, org_name, repo_name, commit_sha):
        commit = self.get_organisation_commit(org_name, repo_name, commit_sha)
        message = commit.commit.message
        if '#' not in message:
            return False
        match = re.search(r"#([0-9]+)", message)
        if match is None:
            return False
        return int(match.group(1))

    def get_origin_pr(self, org_name, repo_name, commit_sha):
        pr_number = self.get_pr_for_commit(org_name, repo_name, commit_sha)
        if not pr_number:
            return False
        pull_request = self.get_organisation_pull_request(org_name, repo_name, pr_number)
        if not pull_request:
            return False
        return pull_request

    def create_release(self, job_name, jira_num, jira_url, version):
        try:
            owner = job_name.split("/")[0]
            repo = job_name.split("/")[1]
            org_repo = self.get_organisation_repository(owner, repo)
            message = f"[{jira_num}]({jira_url}/browse/{jira_num})"
            return org_repo.create_git_release(tag=version, name=version, message=message)
        except GithubException:
            return False



import json
from time import sleep

from reliabilly.components.logger import log_exception, Logger
from reliabilly.components.services import QueryResult, ResultTypes
from reliabilly.components.tools.secret_manager import SecretManager
from reliabilly.components.web.http_requestor import HttpRequestor
from reliabilly.settings import Settings, Constants


@log_exception
def get_sumologic_api_auth_keys(access_id_name, access_key_name, secrets_manager=SecretManager()):
    if Settings.LOCAL_SUMO_API_KEY:
        split_key = Settings.LOCAL_SUMO_API_KEY.split(Constants.COLON)
        return split_key[0], split_key[1]
    api_id = secrets_manager.get_secret(access_id_name)
    api_key = secrets_manager.get_secret(access_key_name)
    return api_id, api_key


# pylint: disable=too-few-public-methods, too-many-arguments
class SumologicQueryExecutor:

    def __init__(self, http=HttpRequestor()):
        self.http = http
        self.logger = Logger(Settings.SERVICE_NAME)
        self.headers = {Constants.CONTENT_TYPE_HEADER: Constants.CONTENT_TYPE_JSON,
                        Constants.ACCEPT_HEADER: Constants.CONTENT_TYPE_JSON}

    def execute_sumologic_query(self, query, from_time, to_time, api_id, api_key):
        try:
            payload = {Constants.QUERY: query, Constants.FROM_DATE_PARAMETER: from_time,
                       Constants.TO_DATE_PARAMETER: to_time}
            self.logger.info(f'Running sumologic query... {payload}')
            result = self.http.post(
                Constants.SUMOLOGIC_QUERY_URL, auth=self.http.get_auth_value(api_id, api_key),
                data=json.dumps(payload, default=str), headers=self.headers)
            self.logger.info(f'Result from sumologic post is... {result.text}')
            if result.status_code == Constants.RECEIVED_SUCCESS_CODE:
                return self.get_sumologic_query_results(result, api_id, api_key)
            return QueryResult(None, ResultTypes.UNKNOWN, Constants.SUMO_NO_RESULT_FOUND_MESSAGE)
        except (Exception, ValueError) as ex:  # pylint: disable=broad-except
            self.logger.error(f'Sumologic query execution failed: {ex}', exc_info=True)
            return QueryResult(None, ResultTypes.ERROR, ex)

    def get_sumologic_query_results(self, query_request_json, api_id, api_key, retry_count=5, wait_time_seconds=5):
        try:
            retries = retry_count
            result_json = query_request_json.json()
            url_get = result_json[Constants.LINK][Constants.HREF]
            self.logger.info(f'Making sumologic get request for query results. {url_get}')
            while retries >= 0:
                retries -= 1
                query_result = self.http.get(
                    url_get, auth=self.http.get_auth_value(api_id, api_key), headers=self.headers)
                if query_result.status_code == Constants.SUCCESS_RESPONSE_CODE:
                    result = query_result.json()
                    message_count = result.get(Constants.SUMOLOGIC_MESSAGE_COUNT, None)
                    if message_count:
                        return QueryResult(message_count, ResultTypes.SUCCESS)
                    return QueryResult(None, ResultTypes.UNKNOWN, Constants.SUMO_MISSING_COUNT_MESSAGE)
                sleep(wait_time_seconds)
            return QueryResult(0, ResultTypes.UNKNOWN, Constants.SUMO_RETRY_EXCEEDED_MESSAGE)
        except (Exception, ValueError) as ex:  # pylint: disable=broad-except
            self.logger.error(f'Error getting sumologic query results. {ex}')
            return QueryResult(None, ResultTypes.ERROR, ex)



from reliabilly.components.chat.slack_client import SlackClient


class ChatClient:
    """ This class is intended to abstract away our chat tool. """
    def __init__(self, chat_client=SlackClient()):
        self.client = chat_client

    # pylint: disable=dangerous-default-value
    def send_room_message(self, room, message, attachments=dict()):
        return self.client.send_message(room, message, attachments=attachments)

    def send_user_message(self, email, message, attachments=dict()):
        return self.client.send_dm(email, message, attachments)

    # pylint: disable=dangerous-default-value
    def send_ephemeral_message(self, room, message, user):
        return self.client.send_ephemeral_message(room, message, user)

    # pylint: disable=dangerous-default-value
    def update_message(self, room, message, timestamp, attachments=dict()):
        return self.client.update_message(room, message, timestamp, attachments)

    def add_reaction(self, room, name, timestamp):
        return self.client.add_reaction(room, name, timestamp)



from nameko.runners import ServiceRunner
from .controllers.jjb_controller import JjbController
from .settings import Constants


RUNNER = ServiceRunner(config=Constants.NAMEKO_CONFIG)
RUNNER.add_service(JjbController)



from datetime import timedelta, datetime

import pendulum


def get_last_hour_epoch():
    return int((datetime.now() - timedelta(hours=1)).timestamp() * 1000)


def get_current_epoch():
    return int(datetime.now().timestamp() * 1000)


def get_last_hour():
    return pendulum.now().add(hours=-1).to_iso8601_string()


def get_current_datetime():
    return pendulum.now().to_iso8601_string()


def get_date_today():
    return pendulum.today().to_iso8601_string()


def daterange(start_date, end_date):
    for day in range(int((end_date - start_date).days)):
        yield start_date + timedelta(day)


def get_date_from_string(date_string):
    return pendulum.parse(date_string)



import json
from threading import Thread

from reliabilly.components.controllers.data_controller import DataController, http, get_route, get_item_route, \
    get_log_string
from reliabilly.components.logger import Logger
from reliabilly.components.monitoring import MonitoringProvider
from reliabilly.settings import Settings, Constants


def perform_collection(collector):
    return collector.perform_collection()


class CollectionController:
    name = Settings.SERVICE_NAME
    monitor = MonitoringProvider()

    def __init__(self, **kwargs):
        self.logger = Logger()
        self.collector = kwargs.get(Constants.COLLECTOR, None)
        self.controller = kwargs.get(Constants.CONTROLLER, DataController(**kwargs))
        self.client = kwargs.get(Constants.CLIENT, None)

    @http(Constants.GET_REQUEST, get_route(Constants.PING_ROUTE))
    def ping(self, request):
        return self.controller.ping(request)

    @http(Constants.OPTIONS_REQUEST, get_route())
    @http(Constants.OPTIONS_REQUEST, get_route(Constants.RUN_ROUTE))
    def options(self, request):
        return self.controller.options(request)

    @http(Constants.OPTIONS_REQUEST, get_item_route())
    def options_item(self, request, item_id):
        return self.controller.options_item(request, item_id)

    @http(Constants.GET_REQUEST, get_route())
    def get(self, request):
        return self.controller.get(request)

    @http(Constants.POST_REQUEST, get_route(Constants.PURGE_ROUTE))
    def purge(self, request):
        return self.controller.purge(request)

    @http(Constants.GET_REQUEST, get_item_route())
    def get_one(self, request, item_id):
        return self.controller.get_one(request, item_id)

    @http(Constants.GET_REQUEST, get_route(Constants.COUNT_ROUTE))
    def count(self, request):
        return self.controller.count(request)

    @http(Constants.GET_REQUEST, get_route(Constants.HEALTH_ROUTE))
    def health(self, request):
        return self.controller.health(request)

    @http(Constants.GET_REQUEST, get_route(Constants.VERSION_ROUTE))
    def version(self, request):
        return self.controller.version(request)

    @http(Constants.PUT_REQUEST, get_route())
    @http(Constants.DELETE_REQUEST, get_route())
    def bad_verbs(self, request):
        return self.controller.bad_verbs(request)

    @http(Constants.POST_REQUEST, get_route())
    def post(self, request):
        self.logger.info(get_log_string(request, Constants.POST_REQUEST, self.name))
        return Constants.BAD_REQUEST_CODE, Constants.BAD_REQUEST_MESSAGE

    @http(Constants.PUT_REQUEST, get_item_route())
    def put(self, request, item_id):
        self.logger.info(get_log_string(request, Constants.PUT_REQUEST, self.name, item_id))
        return Constants.BAD_REQUEST_CODE, Constants.BAD_REQUEST_MESSAGE

    @http(Constants.POST_REQUEST, get_route(Constants.COLLECTION_ROUTE))
    def collect(self, request):
        return self.collect_post(request, self.collector, self.logger)

    @http(Constants.GET_REQUEST, get_route(Constants.COLLECTION_ROUTE))
    def collect_result(self, request):
        return self.collect_get(request, self.collector, self.logger)

    @http(Constants.DELETE_REQUEST, get_route(Constants.COLLECTION_ROUTE))
    def collect_purge(self, request):
        return self.collect_delete(request, self.collector, self.logger)

    @http(Constants.DELETE_REQUEST, get_item_route())
    def delete(self, request, item_id):
        self.logger.info(get_log_string(request, Constants.DELETE_REQUEST, self.name, item_id))
        return Constants.BAD_REQUEST_CODE, Constants.BAD_REQUEST_MESSAGE

    @staticmethod
    def collect_post(request, collector, logger):
        logger.info(get_log_string(request, Constants.POST_REQUEST, __name__))
        thread = Thread(target=perform_collection, args=(collector,))
        thread.start()
        return Constants.RECEIVED_SUCCESS_CODE, Constants.COLLECTION_RECEIVED_MESSAGE

    @staticmethod
    def collect_get(request, collector, logger):
        logger.info(get_log_string(request, Constants.GET_REQUEST, __name__))
        result = collector.manager.get_collection_status(collector.name)
        if not result:
            return Constants.SUCCESS_RESPONSE_CODE, Constants.NO_COLLECTION_FOUND_MESSAGE
        return Constants.SUCCESS_RESPONSE_CODE, json.dumps(result, default=str)

    @staticmethod
    def collect_delete(request, collector, logger):
        logger.info(get_log_string(request, Constants.DELETE_REQUEST, __name__))
        collector.manager.purge_prior_collections()
        return Constants.SUCCESS_RESPONSE_CODE, Constants.PURGE_REQUEST_MESSAGE



import base64
from reliabilly.tests import MagicMock, TestCase, logging, warnings, json, skipUnless
from reliabilly.components.factories.factory import Factory, Components
from reliabilly.components.services.queuing import SqsReceiver, QueueMessage
from reliabilly.settings import Settings, Constants

TEST = 'test'
DUMMY = 'dummy'
TEST_CONTENT = {'content': TEST}
TEST_MESSAGE = QueueMessage(**TEST_CONTENT)
ENCODED_TEST_MESSAGE = 'eyJtZXNzYWdlIjogeyJjb250ZW50IjogInRlc3QifSwgImlkIjogInRlc3QifQ=='
MOCK_RESPONSE = {Constants.MESSAGES_KEY: [{Constants.BODY_KEY: ENCODED_TEST_MESSAGE,
                                           Constants.RECEIPT_HANDLE_KEY: '123'}]}
DUMMY_MESSAGE = {'content': DUMMY}
ENCODED_DUMMY_MESSAGE = base64.b64encode(json.dumps(DUMMY_MESSAGE).encode())
LOCAL_SQS = 'http://localhost:9324'


class QueueingTests(TestCase):
    def setUp(self):
        logging.disable(logging.CRITICAL)
        warnings.filterwarnings(Constants.IGNORE, category=ResourceWarning)

    def test_full_queue(self):
        mock = MagicMock()
        mock.create_queue.return_value = mock
        mock.get_queue_by_name.return_value = mock
        mock.receive_message.return_value = MOCK_RESPONSE
        client = Factory.create(Components.ServiceClients.SQS, queue_name=DUMMY, client=mock)
        self.assertIsNotNone(client)
        self.assertTrue(client.send_message(TEST_MESSAGE))
        mock.send_message.assert_called()
        message = client.receive_message()
        self.assertTrue(message)
        self.assertTrue(client.complete_message(message))
        mock.send_message.side_effect = Exception
        self.assertFalse(client.send_message(TEST_MESSAGE))
        mock.receive_message.side_effect = Exception
        self.assertFalse(client.receive_message())
        mock.delete_message.side_effect = Exception
        self.assertFalse(client.complete_message(message))

    def test_purge(self):
        mock = MagicMock()
        client = Factory.create(Components.ServiceClients.SQS, queue_name=DUMMY, client=mock)
        self.assertTrue(client.purge())
        mock.purge_queue.side_effect = Exception
        self.assertFalse(client.purge())

    def test_create_queue(self):
        mock = MagicMock()
        mock.get_queue_by_name.side_effect = Exception
        client = Factory.create(Components.ServiceClients.SQS, queue_name=DUMMY, client=mock)
        self.assertTrue(client.create_queue(DUMMY))
        mock.create_queue.side_effect = Exception
        self.assertFalse(client.create_queue(DUMMY))


class QueueReceiverTests(TestCase):
    def setUp(self):
        logging.disable(logging.CRITICAL)
        warnings.filterwarnings(Constants.IGNORE, category=ResourceWarning)

    def test_setup(self):
        mock = MagicMock()
        queue = SqsReceiver(DUMMY, client=mock)
        queue.setup()
        self.assertIsNotNone(queue.client)
        mock.create_queue.assert_called()

    def test_run(self):
        mock = MagicMock()
        queue = SqsReceiver(DUMMY, client=mock)
        queue.exit = True
        self.assertTrue(queue.run(mock))
        self.assertTrue(queue.run())
        queue.container = mock
        queue.start()


class TestQueueMessage(TestCase):
    def test_message_encoding(self):
        message = TEST_MESSAGE
        self.assertIsNotNone(message.body)
        self.assertEqual(len(message.body), 100)  # the encoded length in bytes will always include the changing id

    def test_message_decoding(self):
        message = QueueMessage.create_from_encoded(ENCODED_TEST_MESSAGE)
        self.assertIsNotNone(TEST_MESSAGE.message_id, message.message_id)
        self.assertEqual(TEST_CONTENT, message.message)

    def test_message_construction(self):
        message = QueueMessage(test=2, test2='test', message_id='123')
        self.assertEqual(str(message), "{'test': 2, 'test2': 'test', 'message_id': '123'}")


@skipUnless(Settings.RUN_SKIPPED, Constants.RUN_SKIPPED_MSG)
class TestRealQueue(TestCase):
    def setUp(self):
        logging.disable(logging.CRITICAL)
        warnings.filterwarnings(Constants.IGNORE, category=ResourceWarning)

    def test_queue_lost_message(self):
        client = Factory.create(Components.ServiceClients.SQS, queue_name='test', endpoint_url=LOCAL_SQS)
        message = QueueMessage()
        message.product = "site.reliability"
        message.account_name = "reliabilly_ps"
        message.portfolio = "Platform Services"
        message.threshold = "5"
        message.slo_type = "error_rate"
        message.component = "reliabillyecs_cluster"
        message.query = "SELECT sum(`provider.httpCodeTarget5XXCount.Sum`) from LoadBalancerSample where " \
                        "`label.serverclass` like 'lighthouse' and `provider.awsRegion` = 'us-west-2'"
        message.query_executor = "newrelic"
        message.id = "0000000-ae7a-4896-a957-9b736128eaf3"
        message.name = "lighthouse-error_rate-uat"
        client.send_message(message)
        received_message = client.receive_message()
        self.assertEqual(message.message_id, received_message.message_id)
        client.complete_message(received_message)

    def test_actual_sqs(self):
        client = Factory.create(Components.ServiceClients.SQS, queue_name='test')
        model = QueueMessage()
        model.message = 'This is a test ___'
        model.name = 'ODB'
        model.value = 420
        self.assertTrue(client.send_message(model))
        message = client.receive_message()
        self.assertEqual(message.name, model.name)
        client.complete_message(message)

    def test_actual_queue_message(self):
        client = Factory.create(Components.ServiceClients.SQS, queue_name='qm-queue')
        self.assertTrue(client.send_message(QueueMessage(**DUMMY_MESSAGE)))
        message = client.receive_message()
        self.assertIsNotNone(message)
        client.complete_message(message)



# pylint: disable=too-few-public-methods
import datetime


class SetupSettings:
    REQUIREMENTS_FILE = 'requirements.txt'
    PACKAGE_NAME = 'reliabilly'
    PYTHON_REQUIRES = '~=3.6'
    DESCRIPTION = 'Python common types for reliable microservice development.'
    AUTHOR = 'Corpseware'
    AUTHOR_EMAIL = 'jblouse@corpseware.com'
    EXCLUDE_MODULES = ['markdown', 'tests*']
    VERSION = datetime.datetime.now().strftime('%Y.%m.%d%H%M')
    LICENSE = 'MIT'

    with open(REQUIREMENTS_FILE) as file:
        file_requirements = file.read().split('\n')
        REQUIREMENTS = [line for line in file_requirements if '#' not in line]



from jira import JIRA, JIRAError
from reliabilly.settings import Constants

PROJECT_NAME = 'reliabilly'


def get_default_task_client(user, token, baseurl=Constants.TASK_MANAGEMENT_URL_DEFAULT):
    client = None
    try:
        client = JIRA(baseurl, basic_auth=(user, token))
    except JIRAError as ex:
        print(ex)
        exit(False)

    return client


class TaskManagementClient:
    def __init__(self, user, token, url=Constants.TASK_MANAGEMENT_URL_DEFAULT, client=None):
        self.client = client
        if not self.client:
            self.client = get_default_task_client(user, token, url)

    def close_ticket(self, ticket_number):
        try:
            self.client.transition_issue(ticket_number, "121")
            return True
        except JIRAError:
            return False

    def set_awaiting_review(self, ticket_number):
        try:
            self.client.transition_issue(ticket_number, "101")
            return True
        except JIRAError:
            return False

    @staticmethod
    def get_task_summary(issue):
        summary = issue.fields.summary
        return summary

    @staticmethod
    def get_task_description(issue):
        description = issue.fields.description
        return description

    def comment_issue(self, issue, message):
        try:
            self.client.add_comment(issue, message)
            return True
        except JIRAError:
            return False

    def get_issues_by_user(self, user):
        jira_query = 'project={} and sprint in openSprints() '.format(PROJECT_NAME)
        jira_query += 'and status in ("In Progress")'
        jira_query += 'and assignee in ("' + user + '") '
        return self.client.search_issues(jira_query)



from unittest import TestCase, skipUnless
from unittest.mock import MagicMock
from requests.exceptions import RequestException
from reliabilly.components.tools.version import check_versions, get_service_versions, version, \
    run_version_check, check_version_list
from reliabilly.settings import Settings, Constants


LOCAL_TARGET = 'http://localhost:8080'
LOCAL_DEV_VERSION = 'dev'
SERVICE_LIST = ['portfolios', 'infrastructure']
SERVICE_VERSION_LIST = {'portfolios': LOCAL_DEV_VERSION, 'infrastructure': LOCAL_DEV_VERSION}
TEST_PAYLOAD = '{"service": "infrastructure", "version": "dev", "build-number": "None"}'


class VersionCheckTests(TestCase):
    def test_check_service_versions(self):
        mock = MagicMock()
        mock_result = MagicMock()
        mock_result.text = TEST_PAYLOAD
        mock.get.return_value = mock_result
        result = get_service_versions(Constants.DUMMY, SERVICE_LIST, mock)
        self.assertEqual(len(result), 2)

    def test_check_service_fail(self):
        mock = MagicMock()
        mock.get.side_effect = RequestException
        result = get_service_versions(Constants.DUMMY, SERVICE_LIST, mock)
        self.assertIsNotNone(result)

    def test_check_service_404(self):
        mock = MagicMock()
        mock_result = MagicMock()
        mock_result.return_code = 404
        mock.get.side_effect = mock_result
        result = get_service_versions(Constants.DUMMY, SERVICE_LIST, mock)
        self.assertIsNotNone(result)

    def test_run_version_check(self):
        mock = MagicMock()
        mock_result = MagicMock()
        mock_result.status_code = 200
        mock_result.text = TEST_PAYLOAD
        mock.get.side_effect = RequestException
        result = run_version_check(Constants.DUMMY, Constants.DUMMY, SERVICE_LIST, mock)
        self.assertFalse(result)
        mock.reset_mock()
        mock.get.side_effect = [mock_result, mock_result]
        result = run_version_check(Constants.DUMMY, LOCAL_DEV_VERSION, SERVICE_LIST, mock)
        self.assertTrue(result)

    def test_check_version_success(self):
        mock = MagicMock()
        mock.return_value = True
        result = check_versions(Constants.DUMMY, LOCAL_DEV_VERSION, SERVICE_LIST, 1, mock)
        self.assertTrue(result)

    def test_retry_logic(self):
        mock = MagicMock()
        mock.side_effect = [False, False, False, True]
        result = check_versions(Constants.DUMMY, LOCAL_DEV_VERSION, SERVICE_LIST, 1, mock, .1)
        self.assertTrue(result)
        self.assertEqual(mock.call_count, 4)

    def test_check_version_list(self):
        mock = MagicMock()
        mock.return_value = True
        result = check_version_list(SERVICE_VERSION_LIST, LOCAL_DEV_VERSION)
        self.assertTrue(result)

    def test_version(self):
        self.assertIsNotNone(version())

    @skipUnless(Settings.RUN_SKIPPED, Constants.RUN_SKIPPED_MSG)
    def test_real_version_check(self):
        result = check_versions(LOCAL_TARGET, LOCAL_DEV_VERSION, SERVICE_LIST, 1)
        self.assertTrue(result)



from unittest import TestCase, skipUnless
from unittest.mock import MagicMock
from reliabilly.settings import Constants, Settings
from reliabilly.components.tools.secret_manager import SecretManager


class SecretManagerTests(TestCase):
    def test_get_secret(self):
        mock = MagicMock()
        expected_value = 'test_secret_value'
        mock.get_secret_value.return_value = {'SecretString': expected_value}
        manager = SecretManager(mock, mock)
        actual = manager.get_secret('test')
        self.assertEqual(expected_value, actual)

    def test_get_secret_no_attribute(self):
        mock = MagicMock()
        manager = SecretManager(mock, mock)
        mock.get_secret_value.side_effect = Exception
        result = manager.get_secret('test')
        self.assertIsNone(result)
        mock.error.assert_called_once()

    @skipUnless(Settings.RUN_SKIPPED, Constants.RUN_SKIPPED_MSG)
    def test_real_secret(self):
        manager = SecretManager()
        value = manager.get_secret('reliabilly.developer_keys')
        self.assertEqual(value, 'REPLACE_WITH_EXPECTED')



import json
import pickle
import requests
from requests.auth import HTTPBasicAuth
from reliabilly.settings import Settings, Constants, get_service_url, get_raw_service_url


def save_items(items, json_file, pickle_file):  # pragma: no cover
    with open(pickle_file, 'wb') as handle:
        pickle.dump(items, handle, protocol=pickle.HIGHEST_PROTOCOL)
    with open(json_file, 'w') as file:
        json.dump(items, file)


class HttpRequestor:
    """ Generic web http(s) request client. """
    def __init__(self, **kwargs):
        self.client = kwargs.get(Constants.CLIENT, requests)
        self.save_fn = kwargs.get('save_fn', save_items)
        self.circuit_breaker = kwargs.get(Constants.CIRCUIT_BREAKER, None)

    def perform_web_request(self, verb, endpoint_url, **kwargs):
        if self.circuit_breaker is not None:
            return self.circuit_breaker.breaker.call(self._perform_web_request, verb, endpoint_url, **kwargs)
        return self._perform_web_request(verb, endpoint_url, **kwargs)

    def _perform_web_request(self, verb, endpoint_url, **kwargs):
        return self.client.request(verb, endpoint_url, **kwargs)

    def get(self, endpoint_url, **kwargs):
        return self._perform_web_request(Constants.GET_REQUEST, endpoint_url, **kwargs)

    def post(self, endpoint_url, **kwargs):
        return self._perform_web_request(Constants.POST_REQUEST, endpoint_url, **kwargs)

    def put(self, endpoint_url, **kwargs):
        return self._perform_web_request(Constants.PUT_REQUEST, endpoint_url, **kwargs)

    def delete(self, endpoint_url, **kwargs):
        return self._perform_web_request(Constants.DELETE_REQUEST, endpoint_url, **kwargs)

    def save_data_items(self, service_name, endpoint=Constants.EMPTY, paged=True):
        items = self.get_items(endpoint, paged, service_name)
        endpoint_name = endpoint.strip(Constants.FORWARD_SLASH)
        pickle_file = f'{service_name}{endpoint_name}.pickle'
        json_file = f'backup/{service_name}{endpoint_name}.json'
        self.save_fn(items, json_file, pickle_file)
        return True

    def get_items(self, endpoint, paged, service_name):
        if paged:
            return self.get_all_items(service_name)
        headers = self.get_request_headers()
        url = get_raw_service_url(service_name) + f'{endpoint}'
        return self.get(url, headers=headers).json()

    @staticmethod
    def get_request_headers():
        return {Constants.AUTHORIZATION_HEADER: Settings.AUTH_TOKEN, Constants.SERVICE_NAME_ENV: Settings.SERVICE_NAME}

    def get_all_items(self, service_name):
        url = get_service_url(service_name)
        results = list()
        current_offset = 0
        headers = self.get_request_headers()
        result = self._call_endpoint(url, current_offset, headers=headers)
        results.extend(result[service_name])
        while len(results) < result['total']:
            current_offset += result['limit']
            result = self._call_endpoint(url, current_offset, headers=headers)
            results.extend(result[service_name])
        return results

    def _call_endpoint(self, url, offset, headers=None):
        return self.get(url.format(offset=offset), headers=headers).json()

    @staticmethod
    def get_auth_value(user, password):
        return HTTPBasicAuth(user, password)



from slackclient import SlackClient as SlackApi
from reliabilly.components.logger import _get_logger
from reliabilly.settings import Settings, Constants
from reliabilly.components.web.http_requestor import HttpRequestor


class SlackClient:
    def __init__(self, slack_client=SlackApi(Settings.SLACK_API_TOKEN), web_client=HttpRequestor()):
        self.client = slack_client
        self.web_client = web_client

    # pylint: disable=inconsistent-return-statements
    def get_user_id(self, email):
        email_search = Constants.USERS_EMAIL_QUERY + email
        headers = {Constants.AUTHORIZATION_HEADER: Constants.SHARED_AUTH_TOKEN}
        response = self.web_client.get(email_search, headers=headers)
        data = response.json()
        if data['total'] == 1:
            slack_account = data['users'][0]['slack']
            if slack_account:
                return slack_account['id']

    # pylint: disable=dangerous-default-value
    def send_message(self, channel, message, attachments=dict()):
        result = self.client.api_call(
            Constants.SLACK_POST_MESSAGE_TYPE, channel=channel, text=message, attachments=attachments)
        _get_logger(__name__).debug(result)
        return result.get('ts', result.get('error'))

    # pylint: disable=dangerous-default-value
    def update_message(self, channel, message, timestamp, attachments=dict()):
        result = self.client.api_call(
            Constants.SLACK_UPDATE_MESSAGE_TYPE, channel=channel, text=message, ts=timestamp, attachments=attachments)
        _get_logger(__name__).debug(result)
        return result.get('ts', result.get('error'))

    def send_ephemeral_message(self, channel, message, user):
        result = self.client.api_call(Constants.SLACK_EPHEMERAL_MESSAGE_TYPE, channel=channel, text=message, user=user)
        _get_logger(__name__).debug(result)
        return result.get('ts', result.get('error'))

    def add_reaction(self, channel, name, timestamp):
        result = self.client.api_call(
            Constants.SLACK_ADD_REACTION_TYPE, channel=channel, name=name, timestamp=timestamp)
        _get_logger(__name__).debug(result)
        return result.get('ok')

    # pylint: disable=dangerous-default-value
    def send_dm(self, email, message, attachments=dict()):
        user_id = self.get_user_id(email)
        return self.send_message(user_id, message, attachments)



import unittest
from unittest.mock import MagicMock
from github import GithubException

from reliabilly.components.tools.source_control_client import SourceControlClient


class SourceControlClientTests(unittest.TestCase):  # pylint: disable=R0904

    def test_get_org(self):
        mock = MagicMock()
        mock_response = MagicMock()
        mock_response.email = "methodman@wutangfinancial.biz"
        mock.get_organization.return_value = mock_response
        client = SourceControlClient(mock, mock, mock)
        org = client.get_organisation("wu-tang-financial")
        self.assertEqual(org.email, "methodman@wutangfinancial.biz")

    def test_get_org_failure(self):
        mock = MagicMock()
        mock.get_organization.side_effect = GithubException(404, "Not Found")
        client = SourceControlClient(mock, mock, mock)
        org = client.get_organisation(123)
        self.assertFalse(org)

    def test_get_user(self):
        mock = MagicMock()
        mock_response = MagicMock()
        mock_response.name = "Test User"
        mock.get_user.return_value = mock_response
        client = SourceControlClient(mock, mock, mock)
        user = client.get_user("test-user")
        self.assertEqual(user.name, "Test User")

    def test_get_user_failure(self):
        mock = MagicMock()
        mock.get_user.side_effect = GithubException(404, "Not Found")
        client = SourceControlClient(mock, mock, mock)
        user = client.get_user("deleted-user")
        self.assertFalse(user)

    def test_get_user_repo(self):
        mock = MagicMock()
        mock_repo = MagicMock()
        mock_user = MagicMock()
        mock_repo.url = "https://ice-cold-world.net"
        mock_user.get_repo.return_value = mock_repo
        mock.get_user.return_value = mock_user
        client = SourceControlClient(mock, mock, mock)
        repo = client.get_user_repository("gza", "liquid-swords")
        self.assertEqual(repo.url, "https://ice-cold-world.net")

    def test_get_user_repo_failure(self):
        mock = MagicMock()
        mock_user = MagicMock()
        mock_user.get_repo.side_effect = GithubException(404, "Not Found")
        mock.get_user.return_value = mock_user
        client = SourceControlClient(mock, mock, mock)
        user_repo = client.get_user_repository("mf-doom", "mm.. food")
        self.assertFalse(user_repo)

    def test_get_org_repo(self):
        mock = MagicMock()
        mock_repo = MagicMock()
        mock_org = MagicMock()
        mock_repo.open_issues_count = 0
        mock_org.get_repo.return_value = mock_repo
        mock.get_organization.return_value = mock_org
        client = SourceControlClient(mock, mock, mock)
        org_repo = client.get_organisation_repository("tribe-called-quest", "midnight-marauders")
        self.assertEqual(org_repo.open_issues_count, 0)

    def test_get_org_repo_failure(self):
        mock = MagicMock()
        mock_org = MagicMock()
        mock_org.get_repo.side_effect = GithubException(404, "Not Found")
        mock.get_organization.return_value = mock_org
        client = SourceControlClient(mock, mock, mock)
        org_repo = client.get_organisation_repository("tribe-called-quest", "midnight-marauders")
        self.assertFalse(org_repo)

    def test_get_org_commit(self):
        mock = MagicMock()
        mock_commit = MagicMock()
        mock_repo = MagicMock()
        mock_org = MagicMock()
        commit_sha = "123shasha"
        mock_commit.sha = commit_sha
        mock_repo.get_commit.return_value = mock_commit
        mock_org.get_repo.return_value = mock_repo
        mock.get_organization.return_value = mock_org
        client = SourceControlClient(mock, mock, mock)
        org_commit = client.get_organisation_commit("tribe-called-quest", "midnight-marauders", commit_sha)
        self.assertEqual(org_commit.sha, commit_sha)

    def test_get_org_commit_fail(self):
        mock = MagicMock()
        mock_repo = MagicMock()
        mock_org = MagicMock()
        mock_repo.get_commit.side_effect = GithubException(404, "Not Found")
        mock_org.get_repo.return_value = mock_repo
        mock.get_organization.return_value = mock_org
        client = SourceControlClient(mock, mock, mock)
        org_commit = client.get_organisation_commit("tribe-called-quest", "midnight-marauders", "yahyah")
        self.assertFalse(org_commit)

    def test_get_org_pr(self):
        mock = MagicMock()
        mock_pr = MagicMock()
        mock_repo = MagicMock()
        mock_org = MagicMock()
        mock_pr.title = "did a thing"
        mock_repo.get_pull.return_value = mock_pr
        mock_org.get_repo.return_value = mock_repo
        mock.get_organization.return_value = mock_org
        client = SourceControlClient(mock, mock, mock)
        org_pr = client.get_organisation_pull_request("tribe-called-quest", "midnight-marauders", 123)
        self.assertEqual(org_pr.title, "did a thing")

    def test_get_org_pr_fail(self):
        mock = MagicMock()
        mock_repo = MagicMock()
        mock_org = MagicMock()
        mock_repo.get_pull.side_effect = GithubException(404, "Not Found")
        mock_org.get_repo.return_value = mock_repo
        mock.get_organization.return_value = mock_org
        client = SourceControlClient(mock, mock, mock)
        org_pr = client.get_organisation_pull_request("tribe-called-quest", "midnight-marauders", 123)
        self.assertFalse(org_pr)

    def test_get_pr_number_old_format(self):
        mock = MagicMock()
        mock_repo = MagicMock()
        mock_org = MagicMock()
        mock_commit = MagicMock()
        mock_commit.commit.message = 'Merge pull request #197 from test/master'
        mock_repo.get_commit.return_value = mock_commit
        mock_org.get_repo.return_value = mock_repo
        mock.get_organization.return_value = mock_org
        client = SourceControlClient(mock, mock, mock)
        og_pr_number = client.get_pr_for_commit('blah', 'bleh', 'bluh')
        self.assertEqual(og_pr_number, 197)

    def test_get_pr_number_new_format(self):
        mock = MagicMock()
        mock_repo = MagicMock()
        mock_org = MagicMock()
        mock_commit = MagicMock()
        mock_commit.commit.message = 'New PR format on github very cool (#123)'
        mock_repo.get_commit.return_value = mock_commit
        mock_org.get_repo.return_value = mock_repo
        mock.get_organization.return_value = mock_org
        client = SourceControlClient(mock, mock, mock)
        og_pr_number = client.get_pr_for_commit('blah', 'bleh', 'bluh')
        self.assertEqual(og_pr_number, 123)

    def test_get_pr_number_fail(self):
        mock = MagicMock()
        mock_repo = MagicMock()
        mock_org = MagicMock()
        mock_commit = MagicMock()
        mock_commit.commit.message = 'Not a pr title'
        mock_repo.get_commit.return_value = mock_commit
        mock_org.get_repo.return_value = mock_repo
        mock.get_organization.return_value = mock_org
        client = SourceControlClient(mock, mock, mock)
        og_pr_number = client.get_pr_for_commit('blah', 'bleh', 'bluh')
        self.assertFalse(og_pr_number)

    def test_get_pr_number_no_match(self):
        mock = MagicMock()
        mock_repo = MagicMock()
        mock_org = MagicMock()
        mock_commit = MagicMock()
        mock_commit.commit.message = 'New PR format on github very cool'
        mock_repo.get_commit.return_value = mock_commit
        mock_org.get_repo.return_value = mock_repo
        mock.get_organization.return_value = mock_org
        client = SourceControlClient(mock, mock, mock)
        og_pr_number = client.get_pr_for_commit('blah', 'bleh', 'bluh')
        self.assertFalse(og_pr_number)

    def test_get_origin_pr_old_format(self):
        mock = MagicMock()
        mock_repo = MagicMock()
        mock_org = MagicMock()
        mock_commit = MagicMock()
        mock_commit.commit.message = 'Merge pull request #197 from test/master'
        mock_repo.get_pull.return_value = 'a pr object'
        mock_repo.get_commit.return_value = mock_commit
        mock_org.get_repo.return_value = mock_repo
        mock.get_organization.return_value = mock_org
        client = SourceControlClient(mock, mock, mock)
        origin_pr = client.get_origin_pr('blah', 'bleh', 'shashasha')
        self.assertEqual(origin_pr, 'a pr object')

    def test_get_origin_pr_new_format(self):
        mock = MagicMock()
        mock_repo = MagicMock()
        mock_org = MagicMock()
        mock_commit = MagicMock()
        mock_commit.commit.message = 'New PR format on github very cool (#198)'
        mock_repo.get_pull.return_value = 'a pr object'
        mock_repo.get_commit.return_value = mock_commit
        mock_org.get_repo.return_value = mock_repo
        mock.get_organization.return_value = mock_org
        client = SourceControlClient(mock, mock, mock)
        origin_pr = client.get_origin_pr('blah', 'bleh', 'shashasha')
        self.assertEqual(origin_pr, 'a pr object')

    def test_get_origin_non_pr(self):
        mock = MagicMock()
        mock_repo = MagicMock()
        mock_org = MagicMock()
        mock_commit = MagicMock()
        mock_commit.commit.message = 'I am not pull request!'
        mock_org.get_repo.return_value = mock_repo
        mock.get_organization.return_value = mock_org
        client = SourceControlClient(mock, mock, mock)
        origin_pr = client.get_origin_pr('blah', 'bleh', 'shashasha')
        self.assertEqual(origin_pr, False)

    def test_get_origin_pr_fail(self):
        mock = MagicMock()
        mock_repo = MagicMock()
        mock_org = MagicMock()
        mock_commit = MagicMock()
        mock_commit.commit.message = 'I am not pull request!'
        mock_repo.get_pull.side_effect = GithubException(404, "Not Found")
        mock_org.get_repo.return_value = mock_repo
        mock.get_organization.return_value = mock_org
        client = SourceControlClient(mock, mock, mock)
        origin_pr = client.get_origin_pr('blah', 'bleh', 'shashasha')
        self.assertEqual(origin_pr, False)

    def test_create_release(self):
        mock = MagicMock()
        mock_org = MagicMock()
        mock_repo = MagicMock()
        mock_release = MagicMock()
        mock_release.body = '[ODB-420](http://dogs.com/browse/ODB-420)'
        mock_repo.create_git_release.return_value = mock_release
        mock_org.get_repo.return_value = mock_repo
        mock.get_organization.return_value = mock_org
        client = SourceControlClient(mock, mock, mock)
        release = client.create_release("marcus/platform", "ODB-420", "http://dogs.com", "1.0.0")
        expected = "[ODB-420](http://dogs.com/browse/ODB-420)"
        self.assertEqual(release.body, expected)

    def test_create_release_failure(self):
        mock = MagicMock()
        mock_org = MagicMock()
        mock_repo = MagicMock()
        mock_release = MagicMock()
        mock_release.body = '[ODB-420](http://dogs.com/browse/ODB-420)'
        mock_repo.create_git_release.side_effect = GithubException(422, "Validation Failed")
        mock_org.get_repo.return_value = mock_repo
        mock.get_organization.return_value = mock_org
        client = SourceControlClient(mock, mock, mock)
        release = client.create_release("marcus/platform", "ODB-420", "http://dogs.com", "1.0.0")
        self.assertFalse(release)



# pylint: disable=invalid-name,too-many-arguments
import json
import base64
from glob import glob
from os import path
import boto3
from invoke import task
from bs4 import BeautifulSoup
import yaml
from reliabilly.components.tools.data_monitoring_client import DataMonitoringClient
from reliabilly.components.tools.source_control_client import SourceControlClient
from reliabilly.components.tools.task_management_client import TaskManagementClient
from reliabilly.components.tools.version import check_versions
from reliabilly.components.web.http_requestor import HttpRequestor
from reliabilly.settings import Constants, Settings

GIT_CONTRIB_COMMAND = 'git log --pretty="%an"'
# The following is needed to avoid circular references.
BUILD_COMMAND = 'inv && inv int  && inv down'
# ----------------------------------------------------

ROLE_NAME = 'Developer'
ADD_SECRET_COMMAND = "aws secretsmanager create-secret --name '{secret_name}' --secret-string '{secret_value}' " \
                     "--kms-key-id '{kms_key}' --region '{aws_region}'"
UPDATE_SECRET_COMMAND = "aws secretsmanager update-secret --secret-id '{secret_name}' " \
                        "--secret-string '{secret_value}' --kms-key-id '{kms_key}' --region '{aws_region}'"
DELETE_SECRET_COMMAND = "aws secretsmanager delete-secret --secret-id '{secret_name}' --region '{aws_region}'"
CHECK_ROLE_COMMAND = f'aws iam get-role --role-name {ROLE_NAME}'
SECRET_PREFIX = 'reliabilly.'


@task
def build(ctx):
    """ Run the build command. """
    ctx.run(BUILD_COMMAND)


@task(pre=[build])
def pr(_):
    """ Submit a new pull request """


@task
def check_service_versions(_, version, sleep=Constants.DEFAULT_VERSION_TIMEOUT, target=Constants.DEFAULT_TARGET):
    """ Check the service endpoint versions. """
    match = check_versions(target, version, Settings.SERVICES, sleep)
    if not match:
        print('Version check failed!')
        exit(1)
    print('Version check succeeded!')


@task
def create_github_release(_, job_name, jira_num, jira_url, version, token):
    """ Create a new Github release """
    client = SourceControlClient(str(token))
    client.create_release(job_name, jira_num, jira_url, version)


@task
def create_release_event(_, release_number, release_status, release_environment, run_display_url, git_url, api_key):
    """ Create a new Release Event """
    client = DataMonitoringClient(api_key)
    client.create_event(release_number, release_status, release_environment, run_display_url, git_url)


@task
def close_work_ticket(_, base64auth, ticket_number):
    """ Close out a task management ticket """
    username, password = base64.b64decode(base64auth).decode(Constants.UTF8).split(Constants.COLON)
    client = TaskManagementClient(username, password)
    client.close_ticket(ticket_number)


@task
def add_secret(ctx, secret_id, secret_value, prod=False):
    """ Create a new secret in parameter store using our kms key. """
    run_secrets_command(ctx, ADD_SECRET_COMMAND, secret_id, secret_value, prod)


def encrypt_file(file_path, prod=False):
    kms_key = get_kms_key(prod)
    aws_region = get_env_region(prod)
    secrets = yaml.load(open(file_path))

    items = []
    for item in secrets['keys']:
        to_encrypt = item['value']
        encrypted = boto_encrypt(to_encrypt, kms_key, aws_region)
        item['value'] = encrypted
        items.append(item)

    secrets['keys'] = items

    with open(file_path, 'w') as outfile:
        yaml.dump(secrets, outfile, default_flow_style=False)


def toggle_secrets_in_file(file_path, prod=False, encrypt=True):
    aws_region = get_env_region(prod)
    kms_key = get_kms_key(prod)
    secrets = yaml.load(open(file_path))
    items = []
    for item in secrets['keys']:
        to_toggle = item['value']
        if encrypt:
            toggled = boto_encrypt(to_toggle, kms_key, aws_region)
        else:
            toggled = boto_decrypt(to_toggle, aws_region)
        item['value'] = toggled
        items.append(item)

    secrets['keys'] = items

    with open(file_path, 'w') as outfile:
        yaml.dump(secrets, outfile, default_flow_style=False)


def boto_encrypt(plaintext, kms_key, region):
    client = boto3.client('kms', region_name=region)
    response = client.encrypt(KeyId=kms_key, Plaintext=plaintext)
    output = response["CiphertextBlob"]
    return output


def boto_decrypt(encrypted, region):
    client = boto3.client('kms', region_name=region)
    response = client.decrypt(CiphertextBlob=encrypted)
    return response['Plaintext'].decode('UTF-8')


@task
def encrypt_secrets(ctx, service):
    """ Encrypt all yml files in the secrets directory and
    remove the non-encrypted versions for safety. """
    if not check_role_assumed(ctx):
        return
    for file in get_secrets_files(service):
        print(f'Encrypting file {file}...')
        prod = Constants.PROD in file
        toggle_secrets_in_file(file, prod, encrypt=True)


@task
def decrypt_secrets(ctx, service):
    """ Decrypt all yml files in the secrets directory to allow for edit. """
    if not check_role_assumed(ctx):
        return
    for file in get_secrets_files(service):
        print(f'Decrypting file {file}...')
        prod = Constants.PROD in file
        toggle_secrets_in_file(file, prod, encrypt=False)


def get_secrets_files(service):
    """Get the secrets files for a service or all"""
    current_dir = path.dirname(path.realpath(__file__))
    secrets_dir = path.join(current_dir, 'secrets')
    glob_dir = f'{secrets_dir}/*.yml'
    if service:
        glob_dir = f'{service}/secrets/*.yml'
    return glob(glob_dir, recursive=True)


@task
def update_secret(ctx, secret_id, secret_value, prod=False):
    """ Updates a secret in aws secrets manager. """
    if not check_role_assumed(ctx):
        return
    result = run_secrets_command(ctx, UPDATE_SECRET_COMMAND, secret_id, secret_value, prod, True)
    if not result:
        run_secrets_command(ctx, ADD_SECRET_COMMAND, secret_id, secret_value, prod)


@task
def update_service_secrets(ctx, service):
    if not check_role_assumed(ctx):
        return
    decrypt_secrets(ctx, service)
    for secret_file in get_secrets_files(service):
        if Constants.LOCAL in secret_file:
            # Skip local for now until we move to k8s
            continue
        prod = Constants.PROD in secret_file
        update_secrets_file(ctx, secret_file, prod)
    encrypt_secrets(ctx, service)


@task
def delete_secret(ctx, secret_id, prod=False):
    """ Deletes a secret from aws secrets manager. """
    region = get_env_region(prod)
    full_secret_id = get_secret_id(secret_id)
    command = DELETE_SECRET_COMMAND.format(aws_region=region, secret_name=full_secret_id)
    print(f'Deleting secret {full_secret_id}...')
    ctx.run(command)


@task
def wu(ctx):
    """ Generates the list of Wu-Tang names based on project contributors."""
    names = get_contributors(ctx)
    wu_list = list()
    for name in names:
        wu_name = get_wu_name(name)
        wu_list.append(wu_name)
    encoded_wu = get_encoded_wu_names(wu_list)
    print(json.dumps(encoded_wu))


@task
def wu_me(_, name):
    """ Find out what your wu-tang name and auth token will be. """
    wu_name = get_wu_name(name)
    wu_encoded = get_wu_encoded_name(wu_name)
    print(f'Your wu-tang name is: {wu_name}')
    print(f'Your auth token is: {wu_encoded}')


def update_secrets_file(ctx, secrets_file, prod=False):
    environment = Constants.UAT
    if prod:
        environment = Constants.PROD
    with open(secrets_file, Constants.READ_ONLY) as secret_file:
        secret_config = yaml.load(secret_file)
        print(f'Adding secrets from {secrets_file}...')
        for item in secret_config['keys']:
            print(f'Updating secret {item[Constants.NAME]} for {environment}')
            update_secret(ctx, item[Constants.NAME], item[Constants.VALUE], prod)


def check_role_assumed(ctx):
    try:
        ctx.run(CHECK_ROLE_COMMAND, hide=True)
        return True
    except Exception:  # pylint: disable=broad-except
        print('You must have an active assumed role to run this command!')
    return False


def get_secret_id(secret_id):
    full_secret_id = secret_id
    if SECRET_PREFIX not in secret_id:
        full_secret_id = f'{SECRET_PREFIX}{secret_id}'
    return full_secret_id


def run_secrets_command(ctx, command, secret_id, secret_value, prod=False, update=False):
    region = get_env_region(prod)
    kms_key = get_kms_key(prod)
    full_secret_id = get_secret_id(secret_id)
    command = command.format(aws_region=region, secret_name=full_secret_id, secret_value=secret_value, kms_key=kms_key)
    message = get_log_message(update, full_secret_id)
    print(message)
    return ctx.run(command, hide=True, warn=True).ok


def get_log_message(update, secret_id):
    message = f'Adding secret {secret_id}...'
    if update:
        message = f'Updating secret {secret_id}...'
    return message


def get_kms_key(prod=False):
    if prod:
        return Constants.KMS_KEYS[Constants.PROD_REGION]
    return Constants.KMS_KEYS[Constants.UAT_REGION]


def get_wu_name(name):
    wu_name = 'Unknown'
    url = f'https://wunameaas.herokuapp.com/wuami/{name}'
    web_client = HttpRequestor()
    result = web_client.get(url)
    soup = BeautifulSoup(result.text, 'html.parser')
    for bold_item in soup.find_all('b'):
        wu_name = bold_item.get_text()
    return wu_name


def get_wu_encoded_name(name):
    return str(base64.b64encode(name.encode()), Constants.UTF8)


def get_encoded_wu_names(wu_names):
    encoded_list = list()
    for name in wu_names:
        encoded_list.append(get_wu_encoded_name(name))
    return encoded_list


def get_contributors(ctx):
    names = list()
    result = ctx.run(GIT_CONTRIB_COMMAND, hide=True).stdout
    for line in result.split(Constants.NEWLINE):
        name = line.lower().replace(Constants.HYPHEN, Constants.SPACE)
        if name:
            if name not in names:
                names.append(name)
    return names


def get_env_region(prod):
    if prod:
        return Constants.PROD_REGION
    return Constants.UAT_REGION


@task
def send_build_metrics(_, step, state, token, build_tag):
    return call_velocity_endpoint(step, state, token, build_tag)


def call_velocity_endpoint(step, state, token, build_tag, web_client=HttpRequestor()):
    url = build_velocity_url(step, state, token, build_tag)
    result = web_client.post(url)
    return result.status_code


def build_velocity_url(step, state, token, build_tag):
    """Builds a URL for Developer Velocity"""
    full_url = f'{Settings.DV_URL}{token}/{build_tag}/{state}/{step}'
    print(full_url)
    return full_url



import logging
from unittest import TestCase, skipUnless
from unittest.mock import MagicMock

from reliabilly.components.services import ResultTypes
from reliabilly.settings import Settings, Constants
from reliabilly.components.services.sumologic import SumologicQueryExecutor, get_sumologic_api_auth_keys


class SumologicTests(TestCase):
    def setUp(self):
        logging.disable(logging.CRITICAL)

    def test_execute_sumo_query(self):
        mock, mock_get = MagicMock(), MagicMock()
        mock.status_code = Constants.RECEIVED_SUCCESS_CODE
        mock.json.return_value = {
            "id": "1706867DFC65F3C9",
            "link": {
                "rel": "self",
                "href": "https://api.us2.sumologic.com/api/v1/search/"
                        "jobs/1706867DFC65F3C9"
            }
        }
        mock.post.return_value = mock
        mock_get.status_code = Constants.SUCCESS_RESPONSE_CODE
        mock_get.json.return_value = {Constants.SUMOLOGIC_MESSAGE_COUNT: 5}
        mock.get.return_value = mock_get
        executor = SumologicQueryExecutor(mock)
        self.assertIsNotNone(executor)
        auth_token, app_code = get_sumologic_api_auth_keys(mock, mock, mock)
        result = executor.execute_sumologic_query(
            '_sourceCategory=prod/reliabilly/* error | count',
            '2018-07-06T12:00:00', '2018-07-07T12:00:00',
            auth_token, app_code)
        self.assertEqual(result.value, 5)

    def test_execute_sumo_fail_post(self):
        mock = MagicMock()
        mock.status_code = Constants.BAD_REQUEST_CODE
        mock.post.return_value = mock
        executor = SumologicQueryExecutor(mock)
        self.assertIsNotNone(executor)
        auth_token, app_code = get_sumologic_api_auth_keys(mock, mock, mock)
        result = executor.execute_sumologic_query(
            '_sourceCategory=prod/reliabilly/* error | count',
            '2018-07-06T12:00:00', '2018-07-07T12:00:00',
            auth_token, app_code)
        self.assertIsNone(result.value)

    def test_execute_sumo_fail_error(self):
        mock = MagicMock()
        mock.status_code = Constants.BAD_REQUEST_CODE
        mock.post.side_effect = Exception
        executor = SumologicQueryExecutor(mock)
        self.assertIsNotNone(executor)
        auth_token, app_code = get_sumologic_api_auth_keys(mock, mock, mock)
        result = executor.execute_sumologic_query(
            '_sourceCategory=prod/reliabilly/* error | count',
            '2018-07-06T12:00:00', '2018-07-07T12:00:00',
            auth_token, app_code)
        self.assertIsNone(result.value)

    def test_get_sumo_timer(self):
        mock = MagicMock()
        mock.status_code = Constants.BAD_REQUEST_CODE
        mock.post.side_effect = Exception
        executor = SumologicQueryExecutor(mock)
        self.assertIsNotNone(executor)
        result = executor.get_sumologic_query_results(mock, mock, mock, 1, .1)
        self.assertEqual(ResultTypes.UNKNOWN, result.result)
        self.assertEqual(0, result.value)

    def test_get_sumo_fail(self):
        mock = MagicMock()
        mock.status_code = Constants.BAD_REQUEST_CODE
        mock.get.side_effect = Exception
        executor = SumologicQueryExecutor(mock)
        self.assertIsNotNone(executor)
        result = executor.get_sumologic_query_results(mock, mock, mock)
        self.assertIsNone(result.value)

    @skipUnless(Settings.RUN_SKIPPED, Constants.RUN_SKIPPED_MSG)
    def test_execute_real(self):
        executor = SumologicQueryExecutor()
        auth_token, app_code = get_sumologic_api_auth_keys('', '')
        result = executor.execute_sumologic_query(
            '_sourceCategory=prod/reliabilly/* error | count',
            '2018-07-06T12:00:00', '2018-07-07T12:00:00',
            auth_token, app_code)
        self.assertTrue(result.value > 0)



import json
from reliabilly.settings import Settings
from reliabilly.components.web.http_requestor import HttpRequestor


class NewRelicHttpClient:
    def __init__(self, api_key, url=Settings.DATA_MONITORING_URL):
        self.api_key = api_key
        self.url = url

    def generate_headers(self):
        return {'Content-Type': 'application/json', 'X-Insert-Key': self.api_key}

    def create_event(self, payload, requestor=HttpRequestor()):
        headers = self.generate_headers()
        url = self.url
        proxy = {'http': Settings.PROXY_URL}
        response = requestor.post(url, data=json.dumps(payload), headers=headers, proxies=proxy)
        if response.status_code == 202:
            return True
        return False



from pymongo import MongoClient
from pymongo.errors import ServerSelectionTimeoutError
from bson import ObjectId
from reliabilly.components.data import DataClient, get_query
from reliabilly.components.logger import Logger, log_exception
from reliabilly.components.models.base import BaseModel
from reliabilly.components.monitoring import monitor, MonitoringProvider
from reliabilly.settings import Constants, Settings


class MongoDataClient(DataClient):
    """ Data client for the mongodb nosql document data store. """

    def __init__(self, **kwargs):
        """ Initialization for mongodb with support for TDD. """
        super().__init__(**kwargs)
        self.db_name = Settings.MONGO_DB_NAME
        collection_name = kwargs.get(Constants.TABLE, Settings.SERVICE_NAME)
        self.host = kwargs.get(Constants.HOST, Settings.MONGO_URL)
        self.port = kwargs.get(Constants.PORT, Settings.MONGO_PORT)
        self.client = kwargs.get(Constants.CLIENT, MongoClient(self.host, self.port, serverSelectionTimeoutMS=20))
        self.collection = None
        self.logger = Logger()
        self.database = self.client[self.db_name]
        self._select_collection(collection_name)

    def get_health(self):
        try:
            return self.database.command(Constants.MONGO_STATS_COMMAND)
        except ServerSelectionTimeoutError:
            return {Constants.HEALTH: Constants.MONGO_CONNECTION_HEALTH_ERROR.format(host=self.host, port=self.port)}

    @log_exception
    def _select_collection(self, collection_name):
        collection = self.get_collection_name(collection_name)
        self.collection = self.client[self.db_name][collection]

    @log_exception
    def rename_table(self, old_name, new_name):
        self.logger.debug(f'Rename table called on data client. old: {old_name} new: {new_name}')
        self.database[old_name].rename(new_name, dropTarget=True)

    @log_exception
    def table_exists(self, table_name):
        self.logger.debug(f'Existence check on table: {table_name}')
        return table_name in self.database.collection_names()

    @log_exception
    @monitor(monitor_name=MonitoringProvider.get_monitor_prefix(Settings.SERVICE_NAME, 'mongo_create'))
    def create(self, model):
        if isinstance(model, BaseModel):
            model = model.__dict__
        result = self.collection.insert_one(model)
        return result.inserted_id is not None

    @log_exception
    def restore(self, data):
        self.logger.debug(f'Restoring data...')
        result = self.collection.insert_many(data)
        return result.inserted_ids is not None

    @log_exception
    @monitor(monitor_name=MonitoringProvider.get_monitor_prefix(Settings.SERVICE_NAME, 'mongo_get'))
    def get(self, key):
        self.logger.debug(f'Get item for id: {key}')
        return self.collection.find_one({Constants.DB_ID_KEY: ObjectId(key)})

    @log_exception
    @monitor(monitor_name=MonitoringProvider.get_monitor_prefix(Settings.SERVICE_NAME, 'mongo_update'))
    def update(self, key, model):
        self.logger.debug(f'Update item for id: {key}')
        if isinstance(model, BaseModel):
            model = model.__dict__
            model.pop(Constants.ID)
        return self.collection.find_one_and_replace({Constants.DB_ID_KEY: ObjectId(key)}, model, upsert=True)

    @log_exception
    @monitor(monitor_name=MonitoringProvider.get_monitor_prefix(Settings.SERVICE_NAME, 'mongo_get_specific'))
    def get_specific(self, id_value, collection):
        self.logger.debug(f'Get specific item for id: {id_value} from {collection}')
        self.collection = self.client[self.db_name][collection]
        return self.collection.find_one({Constants.DB_ID_KEY: id_value})

    @log_exception
    @monitor(monitor_name=MonitoringProvider.get_monitor_prefix(Settings.SERVICE_NAME, 'mongo_get_all'))
    def get_all(self):
        self.logger.debug(f'Get all called')
        return self.convert_cursor_to_array(self.collection.find())

    @log_exception
    @monitor(monitor_name=MonitoringProvider.get_monitor_prefix(Settings.SERVICE_NAME, 'mongo_find'))
    def find(self, key, value):
        self.logger.debug(f'Find called key: {key} val: {value}')
        return self.collection.find_one({key: value})

    @log_exception
    @monitor(monitor_name=MonitoringProvider.get_monitor_prefix(Settings.SERVICE_NAME, 'mongo_count'))
    def count(self):
        self.logger.debug('Count called')
        return self.collection.find().count(True)

    @log_exception
    @monitor(monitor_name=MonitoringProvider.get_monitor_prefix(Settings.SERVICE_NAME, 'mongo_purge'))
    def purge(self):
        self.logger.debug('Purge called')
        return self.collection.drop()

    @log_exception
    @monitor(monitor_name=MonitoringProvider.get_monitor_prefix(Settings.SERVICE_NAME, 'mongo_read'))
    def read(self, offset, limit, query):
        self.logger.debug(f'Read called offset: {offset} limit: {limit} query: {query}')
        count = self.collection.find(query).count()
        cursor = self.collection.find(query).skip(offset).limit(limit)
        return self.convert_cursor_to_array(cursor), offset, limit, count

    @log_exception
    @monitor(monitor_name=MonitoringProvider.get_monitor_prefix(
        Settings.SERVICE_NAME, 'mongo_delete'))
    def delete(self, key):
        self.logger.debug(f'Delete called id: {key}')
        result = self.collection.delete_one({Constants.DB_ID_KEY: ObjectId(key)})
        return result.deleted_count == 1

    @log_exception
    def delete_db(self):
        self.logger.debug('Drop db called.')
        self.client.drop_database(self.db_name)

    @log_exception
    def get_server_info(self):
        self.logger.debug('Queries DB for server info')
        return self.client.server_info()

    @staticmethod
    def convert_cursor_to_array(cursor):
        documents = []
        if cursor is not None:
            for document in cursor:
                documents.append(document)
        return documents

    @staticmethod
    def get_collection_name(model):
        if model:
            if isinstance(model, str):
                return model
            return model.__name__
        return 'not_set'

    def get_status(self):
        pass

    def filter(self, **kwargs):
        offset, limit, query = get_query(**kwargs)
        return self.read(offset, limit, query)



from dateutil import relativedelta
import pendulum


class RelativeTimeManager:

    @staticmethod
    def relative_delta(period="months", units=1):
        if period == "months":
            return relativedelta.relativedelta(months=units)
        if period == "days":
            return relativedelta.relativedelta(days=units)
        if period == "hours":
            return relativedelta.relativedelta(hours=units)
        if period == "minutes":
            return relativedelta.relativedelta(minutes=units)
        if period == "seconds":
            return relativedelta.relativedelta(seconds=units)

        return relativedelta.relativedelta()

    @staticmethod
    def relative_delta_to_seconds(current_datetime, offset_datetime):
        current_pendulum = pendulum.parse(current_datetime)
        offset_pendulum = pendulum.parse(offset_datetime)
        delta_p = offset_pendulum.diff(current_pendulum)
        return delta_p.total_seconds()



import json
from unittest import TestCase
from reliabilly.components.health import HealthCheckProvider, HealthProvider
from reliabilly.settings import Constants


class DummyHealthComponent(HealthProvider):
    pass


class HealthTests(TestCase):
    def test_health_register(self):
        health = HealthCheckProvider()
        health.register_dependency(Constants.MONGO_HEALTH_ID, None)
        health.register_dependency(Constants.DYNAMO_HEALTH_ID, None)
        self.assertIsNotNone(health)
        self.assertEqual(health.get_dependency_count(), 2)

    def test_health_construct(self):
        health = HealthCheckProvider(mongo=None, dynamo=None)
        self.assertIsNotNone(health)
        self.assertEqual(health.get_dependency_count(), 2)

    def test_health_check(self):
        health = HealthCheckProvider(mongo=DummyHealthComponent(), dynamo=None)
        result = health.get_dependency_health(Constants.MONGO_HEALTH_ID)
        self.assertEqual(result, {Constants.HEALTH: Constants.HEALTH_UNKNOWN_MESSAGE})
        health_total = health.check_health()
        health_json = json.loads(health_total)
        self.assertEqual(health_json[Constants.MONGO_HEALTH_ID], {Constants.HEALTH: Constants.HEALTH_UNKNOWN_MESSAGE})
        self.assertAlmostEqual(
            health.check_health(), '{"mongo": {"health": "health unknown"}, "dynamo": {"health": "health unknown"}}')



from invoke import task
from reliabilly.settings import Settings


COMMON_INSTALL = 'python reliabilly/setup.py install'
PIP_COMMAND = 'pip install -r requirements.txt'
PYLINT_LINT_CMD = 'pylint --disable=missing-docstring --output-format=parseable ' + Settings.LINT_MODULES
INVALID_SERVICE_MESSAGE = 'No service <{0}> found.'
DOCS_BUILD_CMD = 'ipecac --destination .'


@task
def services(_):
    """ Print a list of available services for operations
    like `inv start -s <service>`."""
    print('The following services are available for actions:')
    for service in Settings.SERVICES:
        print(service)


@task
def pylint(ctx):
    """ Run the pylint linter """
    ctx.run(PYLINT_LINT_CMD)


@task(post=[pylint])
def lint(_):
    """ Run all linters """
    print("Running linting...")


@task
def pip(ctx):
    """ Run pip install of requirements. """
    ctx.run(PIP_COMMAND)


@task
def docs(ctx):
    """ Build Swagger definition and move to app container """
    ctx.run(DOCS_BUILD_CMD)



# pylint: disable=too-many-arguments
from reliabilly.components.tools.newrelic_http_client import NewRelicHttpClient


def get_default_monitoring_client(api_key):
    return NewRelicHttpClient(api_key)


class DataMonitoringClient:
    def __init__(self, api_key, client=None):
        self.client = client
        if not self.client:
            self.client = get_default_monitoring_client(api_key)

    @staticmethod
    def generate_payload(release_number, release_status, release_environment, run_display_url, git_url):
        return {
            "eventType": "ReleaseEvent",
            "environment": release_environment.lower(),
            "build_status": release_status.lower(),
            "release_number": release_number,
            "git_repo": git_url,
            "build_pipeline": run_display_url,
        }

    def create_event(self, release_number, release_status, release_environment, run_display_url, git_url):
        payload = self.generate_payload(release_number, release_status, release_environment, run_display_url, git_url)
        return self.client.create_event(payload)



import json
from invoke import task
from reliabilly.tasks.debug import get_cluster_details, get_env
from reliabilly.settings import Settings, Constants, get_compose_environment

EMPTY = ''
ACCOUNT_NUMBER = '777777777777'
IMAGE_PREFIX = 'reliabilly/'
DEFAULT_ECR_REGION = 'us-west-2'
DEFAULT_ECR_REPO = f'{ACCOUNT_NUMBER}.dkr.ecr.us-west-2.amazonaws.com'
DEFAULT_TASK_ROLE_ARN = f'arn:aws:iam::{ACCOUNT_NUMBER}:role/task-role'
DOCKER_ECR_LOGIN_CMD = 'aws ecr get-login --no-include-email --region us-west-2'
DOCKER_TEST_CMD = 'docker ps | grep CONTAINER'
DOCKER_MESSAGE = 'Docker is required and needs to be running.'
DOCKER_COMPOSE_UP_CMD = f'docker-compose {Settings.COMPOSE_FLAGS} up -d'
DOCKER_COMPOSE_DOWN_CMD = f'docker-compose {Settings.COMPOSE_FLAGS}  down --remove-orphans'
DOCKER_COMPOSE_LOGS_CMD = f'docker-compose {Settings.COMPOSE_FLAGS} logs'
DOCKER_COMPOSE_LOGS_TAIL_CMD = f'docker-compose {Settings.COMPOSE_FLAGS} logs -f'
DOCKER_COMPOSE_SERVICE_LOGS_CMD = '{command} {service}'
DOCKER_BUILD_CMD = 'docker build {3} {1} -t {0}{1}:{2}'
DOCKER_BUILD_AGENT_CMD = 'docker build {0} -t {1} .'
DOCKER_TAG_CMD = 'docker tag {0} {1}'
CREATE_ECR_REPO_CMD = 'aws ecr create-repository --repository-name {repo_name} --region {region}'
ECR_EXISTS_CMD = 'aws ecr describe-repositories --repository-names {repo_name} --region {region}'
DOCKER_PUSH_CMD = 'docker push {0}'
DOCKER_EXEC_CMD = 'docker exec -it {0} sh'
DOCKER_GET_ID_BY_NAME = 'docker ps -q -f name={0}'
DOCKER_PS_CMD = 'docker ps'
DOCKER_PSA_CMD = 'docker ps -a'
DOCKER_LIST_CMD = 'docker ps -q'
DOCKER_LIST_ALL_CMD = 'docker ps -a -q'
DOCKER_IMAGES_CMD = 'docker images'
DOCKER_IMAGES_LIST_ALL_CMD = 'docker images -q'
DOCKER_IMAGES_LIST_CMD = 'docker images -q'
DOCKER_STOP_CMD = 'docker stop {0}'
DOCKER_RM_CMD = 'docker rm {0}'
DOCKER_RMI_CMD = 'docker rmi --force {0}'
DOCKER_VOLUME_PRUNE_CMD = 'docker volume prune -f'
DOCKER_NETWORK_PRUNE_CMD = 'docker network prune -f'
DOCKER_LIST_DANGLING = 'docker images -f "dangling=true" -q'
DOCKER_BUILD_ARGS = ''
DEFAULT_ECS_FAMILY = '{service}'
ECS_REGISTER_TASK_CMD = 'aws ecs register-task-definition --family {family} -task-role-arn {task_role} ' \
                        "--container-definitions '{task_def}'"
ECS_RUN_TASK_CMD = 'aws ecs run-task --cluster {cluster} --task-definition {task_def} --count {count}'
COMMON_TAG_LOCAL = f'{IMAGE_PREFIX}reliabilly:latest'
COMMON_TAG_LOCAL_VERSION = f'{IMAGE_PREFIX}reliabilly:latest'
COMMON_IMAGE = f'{DEFAULT_ECR_REPO}/{COMMON_TAG_LOCAL}'
COMMON_IMAGE_VERSION = f'{DEFAULT_ECR_REPO}/{COMMON_TAG_LOCAL_VERSION}'
DOCKER_PULL_COMMON_CMD = f'docker pull {COMMON_IMAGE}'
DOCKER_PULL_COMMON_VERSION_CMD = f'docker pull {COMMON_IMAGE_VERSION}'
DOCKER_TAG_COMMON_CMD = f'docker tag {COMMON_IMAGE} {COMMON_TAG_LOCAL}'
DOCKER_TAG_COMMON_VERSION_CMD = f'docker tag {COMMON_IMAGE} {COMMON_TAG_LOCAL_VERSION}'
DOCKER_BUILD_COMMON_CMD = f'docker build . -t {COMMON_TAG_LOCAL}'
DOCKER_BUILD_IMAGE_CMD = 'docker build {0} -t {1}'
DOCKER_TAG_ECR_COMMON_CMD = f'docker tag {COMMON_TAG_LOCAL} {COMMON_IMAGE}'
DOCKER_TAG_ECR_COMMON_VERSION_CMD = f'docker tag {COMMON_TAG_LOCAL} {COMMON_IMAGE_VERSION}'
DOCKER_PUSH_COMMON_CMD = f'docker push {COMMON_IMAGE}'
DOCKER_HEALTH_CMD = 'docker inspect --format "{{json .State.Health }}"'
DOCKER_PUSH_COMMON_VERSION_CMD = f'docker push {COMMON_IMAGE_VERSION}'


@task(aliases=['dangling'])
def rmd(ctx):
    """ Remove dangling images from builds with NONE names. """
    print('Removing dangling docker images...')
    run_docker_batch(ctx, DOCKER_LIST_DANGLING, DOCKER_RMI_CMD)


@task(optional=['prefix', 'ecr_repo', 'tag'])
def build(ctx, prefix=IMAGE_PREFIX, ecr_repo=None, tag='latest'):
    """ Run Docker image build for all of our container images. """
    print('Building docker images...')
    docker_args = EMPTY
    if ecr_repo:
        docker_args = DOCKER_BUILD_ARGS
    for svc in Settings.SERVICES:
        ctx.run(DOCKER_BUILD_CMD.format(
            prefix, svc, tag, docker_args))
        if ecr_repo and prefix:
            source_tag = f'{prefix}{svc}:{tag}'
            destination_tag = f'{ecr_repo}/{source_tag}'
            ctx.run(DOCKER_TAG_CMD.format(source_tag, destination_tag))


def run_docker_batch(ctx, list_command, action_command):
    all_containers = ctx.run(list_command).stdout.split('\n')
    for container in all_containers:
        if container.strip():
            ctx.run(action_command.format(container))


@task
def health(ctx, service):
    """ Check the health status of a specified container by service name. """
    try:
        container = ctx.run(DOCKER_GET_ID_BY_NAME.format(service), hide=True).stdout
        result = ctx.run(f'{DOCKER_HEALTH_CMD} {container}', hide=True).stdout
        json_result = json.loads(result)
        health_status = json_result.get('Status', Constants.EMPTY)
        print(f'{service}: {health_status}')
        return health_status == 'healthy'
    except Exception:  # pylint: disable=broad-except
        return False


@task
def ps(ctx):  # pylint: disable=invalid-name
    """ Run the docker ps command from within our invoke tasks. """
    ctx.run(DOCKER_PS_CMD)


@task
def psa(ctx):
    """ Run the docker ps -a command from within our invoke tasks. """
    ctx.run(DOCKER_PSA_CMD)


@task(optional=['all_containers'])
def stop(ctx, all_containers=False):
    """ Run the docker stop command from within our invoke tasks. """
    if all_containers:
        run_docker_batch(ctx, DOCKER_LIST_ALL_CMD, DOCKER_STOP_CMD)
        return
    run_docker_batch(ctx, DOCKER_LIST_ALL_CMD, DOCKER_STOP_CMD)


@task(optional=['all_containers'])
def rm(ctx, all_containers=False):  # pylint: disable=invalid-name
    """ Run the docker rm command from within our invoke tasks. """
    if all_containers:
        run_docker_batch(ctx, DOCKER_LIST_ALL_CMD, DOCKER_RM_CMD)
        return
    run_docker_batch(ctx, DOCKER_LIST_ALL_CMD, DOCKER_RM_CMD)


@task(optional=['all_images'])
def images(ctx, all_images=False):
    """ Run the docker images command from within our invoke tasks. """
    if all_images:
        ctx.run(DOCKER_IMAGES_CMD)
        return
    ctx.run(DOCKER_IMAGES_CMD)


@task(optional=['all_images'])
def rmi(ctx, all_images=False):
    """ Run the docker rmi command from within our invoke tasks. """
    if all_images:
        run_docker_batch(ctx, DOCKER_IMAGES_LIST_ALL_CMD, DOCKER_RMI_CMD)
        return
    run_docker_batch(ctx, DOCKER_IMAGES_LIST_CMD, DOCKER_RMI_CMD)


@task
def sh(ctx, service):  # pylint: disable=invalid-name
    """ Connects to a shell inside one of our service containers. """
    container_id = ctx.run(DOCKER_GET_ID_BY_NAME.format(service)).stdout.strip()
    if container_id:
        ctx.run(DOCKER_EXEC_CMD.format(container_id), pty=True)


@task
def prune(ctx):
    """ Prunes unused docker volumes and networks. """
    print('Pruning unused docker volumes...')
    ctx.run(DOCKER_VOLUME_PRUNE_CMD)
    print('Pruning unused docker networks...')
    ctx.run(DOCKER_NETWORK_PRUNE_CMD)


@task(post=[stop, rm, rmi, prune])
def purge(_):
    """ Purge all docker containers and images. """
    print("Purging docker containers and images...")


@task
def clear(ctx):
    """ Purge all docker containers and images. """
    print("Purging ALL docker containers and images...")
    stop(ctx, all_containers=True)
    rm(ctx, all_containers=True)
    rmi(ctx, all_images=True)


@task
def login(ctx):
    """ Login to the ECR repository in AWS. """
    result = ctx.run(DOCKER_ECR_LOGIN_CMD).stdout
    ctx.run(result)


@task(optional=['ecr_repo'], pre=[login])
def push(ctx, tag, ecr_repo=DEFAULT_ECR_REPO):
    """ Push the docker images to our ECR repository in AWS. """
    build(ctx, IMAGE_PREFIX, ecr_repo, tag)
    for svc in Settings.SERVICES:
        create_ecr_repository(ctx, svc)
        image = f'{ecr_repo}/{IMAGE_PREFIX}{svc}:{tag}'
        print(f'Pushing {image}...')
        print(DOCKER_PUSH_CMD.format(image))
        ctx.run(DOCKER_PUSH_CMD.format(image))


@task(pre=[login])
def pull_common(ctx):
    """ Pull the common container image from our ecr repository. """
    ctx.run(DOCKER_PULL_COMMON_CMD)
    ctx.run(DOCKER_TAG_COMMON_CMD)


@task
def build_common(ctx):
    """ Build the common container image from source. """
    ctx.run(DOCKER_BUILD_COMMON_CMD)


@task
def build_image(ctx, path, name):
    """ Build a docker container image from specified path with name. """
    ctx.run(DOCKER_BUILD_IMAGE_CMD.format(path, name))


@task(pre=[login])
def push_common(ctx):
    """ Push the common container image up to our ecr repository. """
    build_common(ctx)
    ctx.run(DOCKER_TAG_ECR_COMMON_CMD)
    ctx.run(DOCKER_TAG_ECR_COMMON_VERSION_CMD)
    ctx.run(DOCKER_PUSH_COMMON_CMD)
    ctx.run(DOCKER_PUSH_COMMON_VERSION_CMD)


@task
def create_ecr_repository(ctx, svc):
    """ Create a docker container registry in ecr for a specified service. """
    repo_name = f'{IMAGE_PREFIX}{svc}'
    repo_exists = check_repo_exists(ctx, repo_name)
    if not repo_exists:
        print(f'Creating ecr repo {repo_name}...')
        ctx.run(CREATE_ECR_REPO_CMD.format(repo_name=repo_name, region=DEFAULT_ECR_REGION))


def check_repo_exists(ctx, repo_name):
    # pylint: disable=broad-except
    try:
        ctx.run(ECR_EXISTS_CMD.format(repo_name=repo_name, region=DEFAULT_ECR_REGION), hide=True)
        print(f'{repo_name} already exists so skipping...')
        return True
    except Exception:
        return False


@task(pre=[build])
def up(ctx):  # pylint: disable=invalid-name
    """ Run Docker compose up which brings up the compose stack. """
    get_compose_environment()
    ctx.run(DOCKER_COMPOSE_UP_CMD)


@task()
def raw(ctx):
    """ Run Docker compose up which brings up the compose stack.
    RISKY AS NO BUILD FIRST SO USE AT RISK"""
    get_compose_environment()
    ctx.run(DOCKER_COMPOSE_UP_CMD)


@task
def down(ctx):
    """ Run Docker compose down to bring down the compose stack. """
    get_compose_environment()
    ctx.run(DOCKER_COMPOSE_DOWN_CMD)


@task(optional=['service'])
def logs(ctx, service):
    """ Run Docker compose logs with optional service name. """
    get_compose_environment()
    if service:
        ctx.run(DOCKER_COMPOSE_SERVICE_LOGS_CMD.format(command=DOCKER_COMPOSE_LOGS_CMD, service=service))
        return
    ctx.run(DOCKER_COMPOSE_LOGS_CMD)


@task(optional=['service'])
def tail(ctx, service=None):
    """ Run Docker compose logs with the -f follow or tail option. """
    get_compose_environment()
    if service:
        ctx.run(DOCKER_COMPOSE_SERVICE_LOGS_CMD.format(command=DOCKER_COMPOSE_LOGS_TAIL_CMD, service=service))
        return
    ctx.run(DOCKER_COMPOSE_LOGS_TAIL_CMD)


def register_ecs_task(ctx, service):
    task_definition = get_task_definition(service, 'dev', '123', '1')
    task_family = DEFAULT_ECS_FAMILY.format(service=service)
    command = ECS_REGISTER_TASK_CMD.format(
        service=service, family=task_family, task_def=task_definition, task_role=DEFAULT_TASK_ROLE_ARN)
    print(f'Creating ECS task definition for {service}...')
    result = ctx.run(command, hide=True).stdout
    result_json = json.loads(result)
    task_arn = result_json['taskDefinition']['taskDefinitionArn']
    print(f'ECS task for {service} registered. ARN: {task_arn}')
    return task_arn


def deploy_ecs_task(ctx, service, count=1, prod=False):
    cluster = get_cluster_details(get_env(prod))
    task_arn = register_ecs_task(ctx, service)
    command = ECS_RUN_TASK_CMD.format(
        cluster=cluster, count=count, task_def=task_arn)
    print(f'Running ECS task for {service}...')
    result = ctx.run(command, hide=True).stdout
    result_json = json.loads(result)
    print(result_json)
    print(f'ECS task for {service} started.')


def get_task_definition(service, environment, tag, build_number):
    with open(f'{service}/ecs/svc_{service}.json') as task_def:
        task_string = task_def.read().replace('${registry}', DEFAULT_ECR_REPO).replace(
            '${image_tag}', tag).replace('${environment}', environment).replace(
                '${aws_region}', DEFAULT_ECR_REGION).replace('${build_number}', build_number)
        json_def = json.loads(task_string)
        return json.dumps(json_def)


@task(optional=['service'])
def deploy(ctx, service):
    """ Deploy docker container(s) to our cluster. """
    if service:
        deploy_ecs_task(ctx, service)
        return
    for svc in Settings.COMPOSE_LIST:
        deploy_ecs_task(ctx, svc)


@task
def clean(ctx):
    """ Clean up unused docker resources such as
    dangling containers and volumes."""
    rmd(ctx)
    prune(ctx)



from unittest import TestCase
from dateutil import parser
from reliabilly.components.tools.relative_time_manager import RelativeTimeManager


class TestRelativeTimeManager(TestCase):

    def test_defaults(self):
        datetime = parser.parse("2018-05-29 07:48:23.175653")
        result = datetime - RelativeTimeManager.relative_delta()
        self.assertEqual(result.month, 4)

    def test_months(self):
        datetime = parser.parse("2018-05-29 07:48:23.175653")
        result = datetime - RelativeTimeManager.relative_delta("months")
        self.assertEqual(result.month, 4)

    def test_days(self):
        datetime = parser.parse("2018-05-29 07:48:23.175653")
        result = datetime - RelativeTimeManager.relative_delta("days")
        self.assertEqual(result.day, 28)

    def test_hours(self):
        datetime = parser.parse("2018-05-29 07:48:23.175653")
        result = datetime - RelativeTimeManager.relative_delta("hours")
        self.assertEqual(result.hour, 6)

    def test_minutes(self):
        datetime = parser.parse("2018-05-29 07:48:23.175653")
        result = datetime - RelativeTimeManager.relative_delta("minutes")
        self.assertEqual(result.minute, 47)

    def test_seconds(self):
        datetime = parser.parse("2018-05-29 07:48:23.175653")
        result = datetime - RelativeTimeManager.relative_delta("seconds")
        self.assertEqual(result.second, 22)

    def test_bad_period(self):
        datetime = parser.parse("2018-05-29 07:48:23.175653")
        result = datetime - RelativeTimeManager.relative_delta("blerch")
        self.assertEqual(result, datetime)

    def test_to_seconds(self):
        result = RelativeTimeManager.relative_delta_to_seconds("2018-05-29T22:35:10Z", "2018-06-05T04:50:24Z")
        self.assertEqual(result, 540914)



from unittest import TestCase
from unittest.mock import MagicMock

from reliabilly.components.authorization import Authorization, get_acceptable_token_list
from reliabilly.settings import Constants, Settings


TEST = 'test'
DUMMY = 'dummy'


def token_list():
    return [TEST]


class AuthTests(TestCase):
    def test_auth_token(self):
        mock = MagicMock()
        mock.headers = {Constants.AUTHORIZATION_HEADER: TEST}
        auth = Authorization()
        self.assertEqual(auth.get_auth_token(mock), TEST)
        self.assertEqual(auth.get_auth_token_log(mock), TEST[-3:])
        self.assertTrue(auth.is_permitted(mock, token_list))
        mock.headers = {Constants.AUTHORIZATION_HEADER: TEST[:-2]}
        self.assertEqual(auth.get_auth_token_log(mock), Constants.NONE)

    def test_token_list(self):
        mock = MagicMock()
        mock.get_secret.return_value = None
        Settings.ENV = Constants.PROD
        tokens = get_acceptable_token_list(DUMMY, mock)
        self.assertIn(DUMMY, tokens)
        mock.get_secret.return_value = '["test1", "test2"]'
        tokens = get_acceptable_token_list(Constants.EMPTY, mock)
        self.assertNotIn(DUMMY, tokens)
        self.assertIn('test1', tokens)
        self.assertIn('test2', tokens)

    def test_response(self):
        code, _, _2 = Authorization.get_unauthorized_response()
        self.assertEqual(code, Constants.UNAUTHORIZED_CODE)

    def test_bearer_token(self):
        mock = MagicMock()
        mock.headers = {Constants.AUTHORIZATION_HEADER: Constants.BEARER_PREFIX}
        mock.return_value = True
        auth = Authorization()
        self.assertTrue(auth.is_permitted(mock, mock, mock))



# pylint: disable=invalid-name,broad-except
import base64
import json
import uuid
from functools import partial
import botocore
from boto3 import resource, client
from nameko.extensions import Entrypoint
from reliabilly.components.logger import Logger
from reliabilly.settings import Constants, Settings


def get_sqs_client(endpoint_url):
    config = botocore.client.Config(connect_timeout=1, read_timeout=1)
    if not endpoint_url:
        return client(Constants.SQS_CLIENT, region_name=Settings.AWS_REGION, config=config)
    return client(Constants.SQS_CLIENT, endpoint_url=endpoint_url, region_name=Settings.AWS_REGION, config=config)


def get_sqs_resource(endpoint_url):
    config = botocore.client.Config(connect_timeout=1, read_timeout=1)
    if not endpoint_url:
        return resource(Constants.SQS_CLIENT, region_name=Settings.AWS_REGION, config=config)
    return resource(Constants.SQS_CLIENT, endpoint_url=endpoint_url, region_name=Settings.AWS_REGION, config=config)


def get_encoded_string(message: str, encoding=Constants.UTF8) -> str:
    return base64.b64encode(message.encode(encoding)).decode(encoding)


def get_decoded_string(message: str, encoding=Constants.UTF8) -> str:
    message += Constants.EQUAL * ((4 - len(message) % 4) % 4)  # pad the message with = if needed
    return base64.b64decode(message).decode(encoding)


class QueueMessage:  # pylint: disable=too-many-instance-attributes
    receipt_handle: str = Constants.EMPTY
    message_id: str = Constants.EMPTY

    def __init__(self, **kwargs):
        self.__dict__ = kwargs
        if not self.message_id:
            self.message_id = str(uuid.uuid4())

    def __str__(self):
        return self.__repr__()

    def __repr__(self):
        return str(self.__dict__)

    @property
    def body(self):
        return get_encoded_string(json.dumps(self.__dict__, default=str))

    @staticmethod
    def create_from_encoded(encoded_message: str):
        message_string = get_decoded_string(encoded_message)
        message = json.loads(message_string)
        return QueueMessage(**message)


class MessageQueue:
    def __init__(self, **kwargs):
        self.logger = kwargs.get(Constants.LOGGER, Logger())
        self.queue_name = kwargs.get(Constants.QUEUE_NAME, None)
        self.endpoint_url = None
        if not Settings.ENV:
            self.endpoint_url = kwargs.get(Constants.ENDPOINT_URL, f'{Settings.SQS_URL}:{Settings.SQS_PORT}')
        self.sqs_resource = kwargs.get(Constants.CLIENT, get_sqs_resource(self.endpoint_url))
        self.sqs_client = kwargs.get(Constants.CLIENT, get_sqs_client(self.endpoint_url))

    def send_message(self, message: QueueMessage):
        try:
            self.logger.info(f'Sending message Id: {message.message_id} Body: {message}')
            sqs_queue = self.create_queue(self.queue_name)
            sqs_queue.send_message(MessageBody=message.body)
            self.logger.info(f'Sent message Id: {message.message_id} Body: {message}')
            return True
        except Exception as ex:
            self.logger.error(f'Error sending message Id: {message.message_id} to queue: {self.queue_name} ex: {ex}')
            return False

    def receive_message(self, wait_seconds: int = Constants.DEFAULT_MESSAGE_WAIT_SECONDS) -> QueueMessage:
        message_id = None
        queue_message = None
        try:
            sqs_queue = self._get_queue(self.queue_name)
            response = self.sqs_client.receive_message(QueueUrl=sqs_queue.url, WaitTimeSeconds=wait_seconds)
            messages = response.get(Constants.MESSAGES_KEY, ())
            for message in messages:
                message_body = message[Constants.BODY_KEY]
                self.logger.info(f'Received raw message from queue {message_body}.')
                queue_message = QueueMessage.create_from_encoded(message_body)
                queue_message.receipt_handle = message[Constants.RECEIPT_HANDLE_KEY]
                message_id = queue_message.message_id
                self.logger.info(f'Received message Id:{message_id} from queue {self.queue_name}.')
                return queue_message
        except Exception as ex:
            if Constants.TIMEOUT not in str(ex):  # The receive message will timeout when there is no message
                self.logger.error(f'Error receiving message from queue {self.queue_name}. Id:{message_id} ex: {ex}')
        return queue_message

    def complete_message(self, message):
        try:
            sqs_queue = self._get_queue(self.queue_name)
            self.sqs_client.delete_message(QueueUrl=sqs_queue.url, ReceiptHandle=message.receipt_handle)
            return True
        except Exception as ex:
            self.logger.error(f'Error completing message {message} on queue. ex: {ex}')
            return False

    def purge(self):
        try:
            sqs_queue = self._get_queue(self.queue_name)
            self.sqs_client.purge_queue(QueueUrl=sqs_queue.url)
            return True
        except Exception as ex:
            self.logger.error(f'Error purging queue {self.queue_name}. ex: {ex}')
            return False

    def create_queue(self, queue_name):
        if not self.sqs_client or not queue_name:
            return None
        sqs_queue = self._get_queue(queue_name)
        if not sqs_queue:
            try:
                self.logger.info(f'Creating sqs queue {queue_name}...')
                sqs_queue = self.sqs_resource.create_queue(
                    QueueName=queue_name, Attributes=Constants.SQS_CREATION_ATTRIBUTES)
                self.sqs_resource.tag_queue(QueueUrl=sqs_queue.url, Tags=Constants.RESOURCE_TAGS)
            except Exception as ex:
                self.logger.info(f'Error calling create_queue {queue_name}. ex: {ex}')
        return sqs_queue

    def _get_queue(self, queue_name):
        try:
            return self.sqs_resource.get_queue_by_name(QueueName=queue_name)
        except Exception as ex:
            self.logger.info(f'Error calling get_queue_by_name {queue_name}. ex: {ex}')
            return None


# noinspection PyArgumentList
class SqsReceiver(Entrypoint):

    def __init__(self, queue_name, **kwargs):
        self.queue_name = queue_name
        self.runner = True
        self.exit = False
        self.client = kwargs.pop(Constants.CLIENT, MessageQueue(queue_name=self.queue_name))
        self.logger = kwargs.pop(Constants.LOGGER, Logger())
        super().__init__(**kwargs)

    def setup(self):
        try:
            self.client.create_queue(self.queue_name)
        except Exception as ex:
            self.logger.info(f'Setup of sqs entrypoint failed for queue {self.queue_name}. {ex}')

    def start(self):
        self.container.spawn_managed_thread(self.run, identifier=Constants.QUEUE_RECEIVER_ID)

    def run(self, receive_func=None):
        if not receive_func:
            receive_func = self.receive_message
        while self.runner:
            receive_func()
            if self.exit:
                break
        return True

    def receive_message(self):
        try:
            message = self.client.receive_message()
            if message:
                self.handle_message(message)
        except Exception as ex:
            self.logger.error(f'Error receiving queue message in nameko entrypoint. ex: {ex}')

    def handle_message(self, message):
        if not message:
            return
        handle_result = partial(self.handle_result, message)
        args = (message,)
        self.container.spawn_worker(self, args, {}, handle_result=handle_result)

    def handle_result(self, message, _, result, exc_info):
        self.client.complete_message(message)
        return result, exc_info


queue = SqsReceiver.decorator



from datetime import datetime

from reliabilly.components.data.mongo_client import MongoDataClient
from reliabilly.settings import Constants


class CollectionManager:

    def __init__(self, **kwargs):
        self.id_key = kwargs.get(Constants.ID, Constants.DB_ID_KEY)
        self.data_client = kwargs.get(Constants.CLIENT, MongoDataClient(table=Constants.COLLECTION_MANAGER_TABLE))

    def is_collection_running(self, collector_name):
        doc = self.get_collection_record(collector_name)
        if doc:
            if doc[Constants.COLLECTION_STATUS_KEY] != Constants.COLLECTION_STARTED_STATUS:
                self.data_client.delete(doc[self.id_key])
                return False
            return True
        return False

    def collection_started(self, collector_name):
        return self.data_client.create(
            {Constants.NAME: collector_name,
             Constants.STARTED: str(datetime.now()),
             Constants.COLLECTION_STATUS_KEY: Constants.COLLECTION_STARTED_STATUS})

    def collection_completed(self, collector_name, success=False):
        collection_doc = self.get_collection_record(collector_name)
        status = Constants.COLLECTION_FAILED_STATUS
        if success:
            status = Constants.COLLECTION_SUCCESS_STATUS
        self.data_client.delete(collection_doc[self.id_key])
        return self.data_client.create(
            {Constants.NAME: collector_name,
             Constants.STARTED: collection_doc[Constants.STARTED],
             Constants.COMPLETED: str(datetime.now()),
             Constants.COLLECTION_STATUS_KEY: status})

    def get_collection_status(self, collector_name):
        status = self.get_collection_record(collector_name)
        if status:
            if status[Constants.COLLECTION_STATUS_KEY] != Constants.COLLECTION_STARTED_STATUS:
                self.data_client.delete(status[self.id_key])
            return status
        return None

    def get_collection_record(self, collector_name):
        doc = self.data_client.find(Constants.NAME, collector_name)
        if isinstance(doc, list):
            if doc:
                return doc[0]
            return None
        return doc

    def purge_prior_collections(self):
        return self.data_client.purge()



import json
from nameko.web.handlers import HttpRequestHandler
from reliabilly.components.authorization import Authorization
from reliabilly.components.controllers.data_handler import DataHandler
from reliabilly.components.data.mongo_client import MongoDataClient
from reliabilly.components.health import HealthCheckProvider
from reliabilly.components.logger import LoggingProvider
from reliabilly.components.models.base import BaseModel
from reliabilly.components.monitoring import MonitoringProvider
from reliabilly.settings import Constants, Settings
from reliabilly.components.tools.version import version


class ExtendedHttpRequestHandler(HttpRequestHandler):
    def __init__(self, *args, **kwargs):
        self.original_request = None
        super().__init__(*args, **kwargs)

    def handle_request(self, request):
        self.original_request = request
        return super().handle_request(request)

    # pylint: disable=arguments-differ
    def response_from_result(self, *args, **kwargs):
        response = super(ExtendedHttpRequestHandler, self).response_from_result(*args, **kwargs)
        original_request = self.original_request
        if not Authorization.is_permitted(original_request):
            unauthorized_result = Authorization.get_unauthorized_response()
            response.set_data(unauthorized_result[2])
            response.status_code = unauthorized_result[0]
        response.headers.add('Access-Control-Allow-Origin', '*')
        response.headers.add('Access-Control-Allow-Credentials', 'true')
        response.headers.add('Access-Control-Allow-Methods', '*')
        response.headers.add('Access-Control-Allow-Headers', 'Origin, X-Requested-With, '
                             'Content-Type, Accept, Authorization')
        response.headers['Content-Type'] = 'application/json'
        return response


# pylint: disable=invalid-name
http = ExtendedHttpRequestHandler.decorator


def get_route(item=''):
    return f'/{Settings.SERVICE_NAME}/{item}'


def get_specific_item_route(sub_route):
    return get_route(f'{sub_route}<string:item_id>')


def get_item_route():
    return get_route('<string:item_id>')


# noinspection PyMethodParameters
class Meta(type):
    def __new__(cls, name, bases, dct):
        var = super().__new__(cls, name, bases, dct)
        var.name = Settings.SERVICE_NAME
        return var


def get_request_ip(request):
    return request.environ.get('HTTP_X_REAL_IP', request.remote_addr)


def get_log_string(request, method, name, item=Constants.EMPTY):
    return f'{name} {method} {item} {request.base_url} ' \
           f'{request.query_string} from {get_request_ip(request)} ' \
           f'auth {Authorization.get_auth_token_log(request)} ' \
           f'route {Constants.COMMA.join(request.access_route)} called...'


class DataController(metaclass=Meta):
    name = Settings.SERVICE_NAME
    model = BaseModel()
    monitor = MonitoringProvider()
    logger = LoggingProvider(name)

    def __init__(self, **kwargs):
        self.client = kwargs.get(Constants.CLIENT, MongoDataClient(table=self.name))
        self.model = kwargs.get(Constants.MODEL, BaseModel())
        self.data_handler = DataHandler(self.client)
        self.healthcheck = HealthCheckProvider(mongo=self.client)

    # pylint: disable=no-self-use
    @http(Constants.OPTIONS_REQUEST, get_route())
    def options(self, _):
        return Constants.SUCCESS_RESPONSE_CODE, version()

    # pylint: disable=no-self-use
    @http(Constants.OPTIONS_REQUEST, get_item_route())
    def options_item(self, _, item_id):
        return Constants.SUCCESS_RESPONSE_CODE, item_id

    @http(Constants.GET_REQUEST, get_route())
    def get(self, request):
        self.logger.info(get_log_string(request, Constants.GET_REQUEST, self.name))
        result = self.data_handler.handle_get_request(request)
        return result

    @http(Constants.GET_REQUEST, get_item_route())
    def get_one(self, request, item_id):
        self.logger.info(get_log_string(request, Constants.GET_REQUEST, self.name))
        return self.data_handler.handle_get_request_single(item_id)

    # pylint: disable=no-self-use
    @http(Constants.GET_REQUEST, get_route(Constants.PING_ROUTE))
    def ping(self, _):
        return Constants.SUCCESS_RESPONSE_CODE, Constants.PING_RESPONSE

    @http(Constants.GET_REQUEST, get_route(Constants.COUNT_ROUTE))
    def count(self, request):
        self.logger.info(get_log_string(request, Constants.GET_REQUEST, self.name))
        return self.data_handler.handle_count_request()

    @http(Constants.GET_REQUEST, get_route(Constants.HEALTH_ROUTE))
    def health(self, request):
        self.logger.info(get_log_string(request, Constants.GET_REQUEST, self.name, Constants.HEALTH))
        return Constants.SUCCESS_RESPONSE_CODE, self.healthcheck.check_health()

    @http(Constants.GET_REQUEST, get_route(Constants.VERSION_ROUTE))
    def version(self, request):
        self.logger.info(get_log_string(request, Constants.GET_REQUEST, self.name))
        return Constants.SUCCESS_RESPONSE_CODE, version()

    @http(Constants.POST_REQUEST, get_route())
    def post(self, request):
        self.logger.info(get_log_string(request, Constants.POST_REQUEST, self.name))
        payload = json.loads(request.get_data(as_text=True))
        if self.model.validate(payload):
            return self.data_handler.handle_post_request(payload)
        return Constants.UNPROCESSABLE_ENTITY_CODE, Constants.UNPROCESSABLE_ENTITY_MESSAGE

    @http(Constants.PUT_REQUEST, get_item_route())
    def put(self, request, item_id):
        self.logger.info(get_log_string(request, Constants.PUT_REQUEST, self.name))
        payload = json.loads(request.get_data(as_text=True))
        return self.data_handler.handle_update_request(item_id, payload)

    @http(Constants.POST_REQUEST, get_route(Constants.RESTORE_ROUTE))
    def restore(self, request):
        self.logger.info(get_log_string(request, Constants.POST_REQUEST, self.name))
        payload = json.loads(request.get_data(as_text=True))
        return self.data_handler.handle_restore_request(payload)

    @http(Constants.PUT_REQUEST, get_route())
    @http(Constants.DELETE_REQUEST, get_route())
    def bad_verbs(self, request):
        self.logger.info(get_log_string(request, Constants.PUT_REQUEST, self.name))
        return Constants.BAD_REQUEST_CODE, Constants.BAD_REQUEST_MESSAGE

    @http(Constants.DELETE_REQUEST, get_item_route())
    @http(Constants.DELETE_REQUEST, get_route())
    def delete(self, request, item_id):
        self.logger.info(get_log_string(request, Constants.DELETE_REQUEST, self.name, item_id))
        self.logger.info(f'{self.name} DELETE {item_id} called...')
        return self.data_handler.handle_delete_request(item_id)

    @http(Constants.POST_REQUEST, get_route(Constants.PURGE_ROUTE))
    def purge(self, request):
        self.logger.info(get_log_string(request, Constants.POST_REQUEST, self.name))
        return self.data_handler.handle_purge_request()



# pylint: disable=too-few-public-methods,invalid-name
import json
from cerberus import Validator
from reliabilly.components.logger import Logger
from reliabilly.settings import Constants


class Field:
    def __init__(self, var_type, **kwargs):
        self.type = var_type
        self.schema = {}
        for key, value in kwargs.items():
            self.schema[key] = value


SchemaTypeMappings = {
    "bool": "boolean",
    "bytes": "binary",
    "datetime.date": "date",
    "datetime.datetime": "datetime",
    "dict": "dict",
    "float": "float",
    "int": "integer",
    "list": "list",
    "set": "set",
    "str": "string"
}


class BaseModel:
    logger = Logger()
    id: str  # pylint: disable=invalid-name

    def __init__(self, dictionary=None):
        if dictionary:
            vars(self).update(dictionary)

    def __repr__(self):
        return self.__str__()

    def __str__(self):
        return self.to_json(self)

    # pylint: disable=inconsistent-return-statements
    @staticmethod
    def safe_set_attribute(attribute):
        if attribute:
            return attribute

    def get_attribute(self, attribute_name):
        return self.__dict__.get(attribute_name, Constants.EMPTY)

    @staticmethod
    def from_json(json_string):
        return json.loads(json_string, object_hook=BaseModel)

    def get_json(self):
        return BaseModel.to_json(self)

    @staticmethod
    def to_json(model):
        return json.dumps(model.__dict__, default=str)

    def validate(self, model=None, schema=None):
        if schema is None:
            schema = self.to_schema()
            if not schema:
                return True
        validator = Validator(schema)
        model_data = self.__dict__
        if model:
            model_data = model
        result = validator.validate(model_data)
        self.logger.info(f'Validation errors: {validator.errors}')
        return result

    def to_schema(self):
        schema = {}
        # pylint: disable=no-member
        functions = self.__annotations__
        for key, value in functions.items():
            # pylint: disable=unidiomatic-typecheck
            if type(value) is not Field:
                return False
            schema[key] = {'type': SchemaTypeMappings[value.type.__name__],}
            schema[key].update(value.schema)
        return schema



# pylint: disable=too-many-arguments
import json
from time import sleep
from requests.exceptions import RequestException
from reliabilly.components.web.http_requestor import HttpRequestor
from reliabilly.settings import Constants, Settings


VERSION_ENDPOINT = f'/{Constants.VERSION_KEY}/'


def build_from_template(template, **kwargs):
    return template.format(**kwargs)


def get_version_from_payload(payload):
    print(payload.text)
    return json.loads(payload.text).get(Constants.VERSION_KEY, None)


def get_service_versions(target_url, service_list, web_client=HttpRequestor()):
    versions = dict()
    for service in service_list:
        url = f'{target_url.strip(Constants.URL_SEPARATOR)}/{service}{VERSION_ENDPOINT}'
        try:
            result = web_client.get(url)
            if result.status_code == Constants.SUCCESS_RESPONSE_CODE:
                versions[service] = get_version_from_payload(result)
            else:
                versions[service] = None
        except RequestException:
            versions[service] = None
    return versions


def check_version_list(versions, expected_version):
    print(versions)
    versions_pass = True
    for _, read_version in versions.items():
        versions_pass &= expected_version == read_version
    return versions_pass


def run_version_check(target_url, expected_version, service_list, web_client=HttpRequestor()):
    versions = get_service_versions(target_url, service_list, web_client)
    return check_version_list(versions, expected_version)


# pylint: disable=too-many-arguments
def check_versions(target_url, expected_version, service_list, timeout=Constants.VERSION_CHECK_TIMEOUT,
                   version_func=run_version_check, sleep_time=5):
    total_time = 0
    result = version_func(target_url, expected_version, service_list)
    while not result:
        print(f'Sleeping {sleep_time} for service cut over...')
        sleep(sleep_time)
        total_time += sleep_time
        result = version_func(target_url, expected_version, service_list)
        if result:
            return result
        if total_time >= timeout:
            return False
    return result


def version():
    return build_from_template(Constants.VERSION_RESPONSE_TEMPLATE, service=Settings.SERVICE_NAME,
                               version=Settings.VERSION, build_number=Settings.BUILD_NUMBER)



# pylint: disable=too-few-public-methods
class Payload:
    @staticmethod
    def get(base, suffix):
        try:
            string = Payload.__getattribute__(Payload, f'{base}{suffix}')
        except AttributeError:
            string = None
        return string

    ALL_TEXT = 'Debug for great success'
    ALL_TITLE = 'Very nice'
    CODE_QUALITY_WARNING_TEXT = "Unfortunately, your pull request has " \
                                "failed during the linting and testing" \
                                " phase. You'll need to fix any issues " \
                                "in order for the team to review your changes."
    CODE_QUALITY_WARNING_TITLE = "{0} has failed linting and testing."
    CODE_QUALITY_SUCCESS_TEXT = "A new pull request has successfully passed" \
                                " testing and is ready to be reviewed by a" \
                                " member of the team."
    CODE_QUALITY_SUCCESS_TITLE = "{0} is ready to be reviewed."
    CODE_QUALITY_FAILURE_MASTER_TEXT = "The master branch has failed to pass" \
                                       " a code quality check. Someone" \
                                       " should investigate how this error" \
                                       " was first introduced."
    CODE_QUALITY_FAILURE_MASTER_TITLE = CODE_QUALITY_WARNING_TITLE
    DEBUG_FAILURE_TEXT = 'Hello! Please ignore me!'
    DEBUG_FAILURE_TITLE = ':very_nice: {0} is being debugged for great success'
    DEBUG_WARNING_TEXT = DEBUG_FAILURE_TEXT
    DEBUG_WARNING_TITLE = DEBUG_FAILURE_TITLE
    DEBUG_SUCCESS_TEXT = DEBUG_FAILURE_TEXT
    DEBUG_SUCCESS_TITLE = DEBUG_FAILURE_TITLE
    DOCKER_BUILD_FAILURE_TITLE = "{0} has failed upon attempting to build " \
                                 "Docker containers"
    DOCKER_PUSH_FAILURE_TITLE = "{0} has failed upon attempting to push to ECR"
    UAT_DEPLOY_FAILURE_TITLE = "{0} has failed while trying to deploy to UAT"
    UAT_DEPLOY_SUCCESS_TITLE = "{0} has successfully deployed to UAT"
    UAT_DEPLOY_SUCCESS_TEXT = "The latest UAT deployment was successful. " \
                              "It's almost to production! :thonking:"
    UAT_INTEGRATION_FAILURE_TITLE = "{0} has failed during integration " \
                                    "testing on UAT"
    UAT_CREATE_SYNTH_FAILURE_TITLE = "{0} has failed while trying to " \
                                     "create UAT synthetic monitors " \
                                     "for New Relic"
    UAT_CREATE_SYNTH_FAILURE_TEXT = "Unfortunately something went wrong " \
                                    "while trying to attach monitors to the" \
                                    " latest UAT deployment."
    PROD_BACKUP_FAILURE_TITLE = "{0} has failed while trying to back up" \
                                " production data"
    PROD_DEPLOY_FAILURE_TITLE = "{0} has failed while trying to deploy to " \
                                "production"
    PROD_DEPLOY_SUCCESS_TITLE = ":very_nice: {0} has successfully deployed " \
                                "to production"
    PROD_DEPLOY_SUCCESS_TEXT = "Aww yeah, your latest pull request has " \
                               "safely made it all the way to production.\n" \
                               "Nice work! :b1:"
    PROD_CREATE_SYNTH_FAILURE_TITLE = "{0} has failed while trying to " \
                                      "create production synthetic monitors " \
                                      "for New Relic"
    PROD_CREATE_SYNTH_FAILURE_TEXT = "Unfortunately something went wrong " \
                                     "while trying to attach monitors to the" \
                                     " latest production deployment."
    PROD_RESTORE_FAILURE_TITLE = "{0} has failed while trying to restore" \
                                 " production backups"
    PROD_INTEGRATION_FAILURE_TITLE = "{0} has failed during integration " \
                                     "testing on production"
    LATEST_COMMIT_URL = '{0}/commit/{1}'
    VIEW_GITHUB_LABEL = ":github: View pull request on Github"
    VIEW_LATEST_COMMIT_LABEL = ":github: View latest commit"
    VIEW_PIPELINE_LABEL = ":jenkins: View Jenkins pipeline"
    VIEW_PROD_LABEL = ":do_it_live: View production environment"
    VIEW_UAT_LABEL = ":uat: View UAT environment"
    SHOW_CHANGE_AUTHOR = ['CODE_QUALITY_SUCCESS',
                          'CODE_QUALITY_FAILURE_MASTER',
                          'DOCKER_BUILD_FAILURE', 'DOCKER_PUSH_FAILURE',
                          'UAT_DEPLOY_FAILURE', 'UAT_INTEGRATION_FAILURE',
                          'PROD_BACKUP_FAILURE', 'PROD_RESTORE_FAILURE',
                          'PROD_INTEGRATION_FAILURE', 'ALL']
    SHOW_PULL_REQUEST_TITLE = ['CODE_QUALITY_SUCCESS',
                               'PROD_DEPLOY_SUCCESS', 'ALL']
    SHOW_LATEST_COMMIT_BUTTON = ['CODE_QUALITY_FAILURE_MASTER',
                                 'DOCKER_BUILD_FAILURE',
                                 'DOCKER_PUSH_FAILURE',
                                 'UAT_DEPLOY_FAILURE',
                                 'UAT_INTEGRATION_FAILURE',
                                 'PROD_CREATE_SYNTH_FAILURE',
                                 'PROD_BACKUP_FAILURE',
                                 'PROD_RESTORE_FAILURE',
                                 'PROD_INTEGRATION_FAILURE', 'ALL']
    SHOW_PULL_REQUEST_BUTTON = ['CODE_QUALITY_WARNING',
                                'CODE_QUALITY_SUCCESS', 'ALL']
    SHOW_PIPELINE_BUTTON = ['CODE_QUALITY_WARNING',
                            'CODE_QUALITY_FAILURE_MASTER',
                            'DOCKER_BUILD_FAILURE', 'DOCKER_PUSH_FAILURE',
                            'UAT_DEPLOY_FAILURE', 'UAT_INTEGRATION_FAILURE',
                            'PROD_BACKUP_FAILURE', 'PROD_DEPLOY_FAILURE',
                            'PROD_RESTORE_FAILURE',
                            'PROD_CREATE_SYNTH_FAILURE',
                            'PROD_INTEGRATION_FAILURE',
                            'PROD_DEPLOY_SUCCESS', 'ALL']
    SHOW_PROD_BUTTON = ['PROD_INTEGRATION_FAILURE', 'PROD_DEPLOY_SUCCESS',
                        'ALL']
    SHOW_UAT_BUTTON = ['UAT_INTEGRATION_FAILURE', 'UAT_DEPLOY_SUCCESS', 'ALL']


class MessageBuilder:
    def __init__(self, message_type, environment):
        self.message_type = message_type
        self.env = environment

    def make_attachment(self):
        attachment = dict()
        fields = list()
        actions = list()

        job_name = self.env.get('JOB_NAME')
        repo_and_branch = ' / '.join(job_name.split('/')[1:])

        attachment['title'] = Payload.get(
            self.message_type, '_TITLE').format(repo_and_branch)

        if Payload.get(self.message_type, '_TEXT'):
            attachment['text'] = Payload.get(self.message_type, '_TEXT')

        if 'FAILURE' in self.message_type:
            attachment['color'] = 'danger'
        if 'WARNING' in self.message_type:
            attachment['color'] = 'warning'
        if 'SUCCESS' in self.message_type:
            attachment['color'] = 'good'

        if self.message_type in Payload.SHOW_CHANGE_AUTHOR:
            fields.append({
                'title': 'Author',
                'value': self.env.get('CHANGE_AUTHOR_DISPLAY_NAME'),
                'short': True
            })

        if self.message_type in Payload.SHOW_PULL_REQUEST_TITLE:
            fields.append({
                'title': 'Description',
                'value': self.env.get('CHANGE_TITLE'),
                'short': True
            })

        if self.message_type in Payload.SHOW_PULL_REQUEST_BUTTON:
            actions.append({
                'type': 'button',
                'text': Payload.VIEW_GITHUB_LABEL,
                'url': self.env.get('CHANGE_URL')
            })

        if self.message_type in Payload.SHOW_LATEST_COMMIT_BUTTON:
            actions.append({
                'type': 'button',
                'text': Payload.VIEW_LATEST_COMMIT_LABEL,
                'url': Payload.LATEST_COMMIT_URL.format(
                    self.env.get('GIT_URL')[0:-4], self.env.get('GIT_COMMIT'))
            })

        if self.message_type in Payload.SHOW_PIPELINE_BUTTON:
            actions.append({
                'type': 'button',
                'text': Payload.VIEW_PIPELINE_LABEL,
                'url': self.env.get('RUN_DISPLAY_URL')
            })

        if self.message_type in Payload.SHOW_UAT_BUTTON:
            actions.append({
                'type': 'button',
                'text': Payload.VIEW_UAT_LABEL,
                'url': self.env.get('UAT_INT_TEST_TARGET')
            })

        if self.message_type in Payload.SHOW_PROD_BUTTON:
            actions.append({
                'type': 'button',
                'text': Payload.VIEW_PROD_LABEL,
                'url': self.env.get('PROD_INT_TEST_TARGET')
            })

        attachment['actions'] = actions
        attachment['fields'] = fields
        return [attachment]



from unittest import TestCase
from unittest.mock import MagicMock

from reliabilly.components.chat.chat_client import ChatClient
from reliabilly.settings import Constants

LIVE_TEST_CHANNEL = 'automation-test'
LIVE_TEST_MESSAGE = 'Message generated from unit test run for ChatClient tests.'
LIVE_TEST_EMAIL = 'jared.blouse@reliabilly.com'


class ChatClientTests(TestCase):
    def test_send_room_message(self):
        mock = MagicMock()
        chat_client = ChatClient(mock)
        self.assertTrue(chat_client.send_room_message(Constants.DUMMY, Constants.DUMMY))
        self.assertTrue(mock.send_message.called)

    def test_send_user_message(self):
        mock = MagicMock()
        chat_client = ChatClient(mock)
        self.assertTrue(chat_client.send_user_message(Constants.DUMMY, Constants.DUMMY))
        self.assertTrue(mock.send_dm.called)

    def test_update_message(self):
        mock = MagicMock()
        chat_client = ChatClient(mock)
        self.assertTrue(chat_client.update_message(Constants.DUMMY, Constants.DUMMY, Constants.DUMMY, Constants.DUMMY))
        self.assertTrue(mock.update_message.called)

    def test_send_ephemeral_message(self):
        mock = MagicMock()
        chat_client = ChatClient(mock)
        self.assertTrue(chat_client.send_ephemeral_message(Constants.DUMMY, Constants.DUMMY, Constants.DUMMY))
        self.assertTrue(mock.send_ephemeral_message.called)

    def test_add_reaction(self):
        mock = MagicMock()
        chat_client = ChatClient(mock)
        self.assertTrue(chat_client.add_reaction(Constants.DUMMY, Constants.DUMMY, Constants.DUMMY))
        self.assertTrue(mock.add_reaction.called)



import json
import logging
import logging.handlers
from datetime import datetime
from os import environ
from reliabilly.components.web.http_requestor import HttpRequestor
from reliabilly.settings import Settings, Constants

LOG_FILE_ENV = 'LOGFILE'
LOG_FILE_DEFAULT = f'{Settings.SERVICE_NAME}.log'
LOG_FORMAT = '%(asctime)s - %(name)s - %(levelname)s - %(message)s'
LOG_LEVEL_ENV = 'LOGLEVEL'
LOG_LEVEL_DEFAULT = 'ERROR'


class PingLogFilter(logging.Filter):  # pylint: disable=too-few-public-methods
    def filter(self, record):
        return 'ping' not in record.getMessage()


def _get_logger(module_name):
    logging.basicConfig(level=environ.get(LOG_LEVEL_ENV, LOG_LEVEL_DEFAULT), format=LOG_FORMAT)
    logger = logging.getLogger(module_name)
    formatter = logging.Formatter(LOG_FORMAT)
    handler = logging.FileHandler(environ.get(LOG_FILE_ENV, LOG_FILE_DEFAULT))
    handler.setFormatter(formatter)
    handler.addFilter(PingLogFilter())
    global_logging = logging.getLogger()
    for handler_def in global_logging.handlers:
        handler_def.addFilter(PingLogFilter())
    logger.addHandler(handler)
    return logger


# noinspection PyBroadException
class SumoLogClient:
    def __init__(self, http_client=HttpRequestor(), log_url=Settings.REMOTE_LOG_URL):
        self.client = http_client
        self.log_url = log_url

    @staticmethod
    def get_environment(region=Settings.AWS_REGION):
        environment = Constants.UAT
        if region == Constants.PROD_REGION:
            environment = Constants.PROD
        return environment

    def get_headers(self, service):
        account = Settings.AWS_ACCOUNT
        environment = self.get_environment()
        version = Settings.VERSION
        return {
            Constants.SUMO_CATEGORY_HEADER_KEY: Constants.SUMO_CATEGORY,
            Constants.SUMO_HOST_HEADER_KEY: Constants.SUMO_HOST_HEADER.format(
                account=account, environment=environment, version=version, service=service),
            Constants.CONTENT_TYPE_HEADER: Constants.CONTENT_TYPE_JSON}

    @staticmethod
    def get_log_message(level, service, message):
        log_data = {Constants.SUMO_MESSAGE_KEY: message,
                    Constants.SUMO_LEVEL_KEY: level,
                    Constants.SERVICE: service,
                    Constants.SUMO_TIMESTAMP_KEY: datetime.now().strftime(Constants.SUMO_TIMESTAMP_FORMAT)}
        return json.dumps(log_data, default=str)

    def ship_log(self, level, service, message):
        try:
            header = self.get_headers(service)
            log_message = self.get_log_message(level, service, message)
            result = self.client.post(Settings.REMOTE_LOG_URL, data=log_message, headers=header)
            return result.status_code == 200
        except Exception:  # pylint: disable=broad-except
            return False


class Logger:
    logger = _get_logger(Settings.SERVICE_NAME)

    def __init__(self, service_name=Settings.SERVICE_NAME, logger=None, remote_client=SumoLogClient()):
        self.name = service_name
        self.log_name = service_name
        self.remote_log = remote_client
        if logger:
            self.logger = logger

    def debug(self, message, *args, **kwargs):
        self.logger.debug(message, *args, **kwargs)
        self.remote_log.ship_log(Logger.debug.__name__, self.name, message)

    def error(self, message, *args, **kwargs):
        self.logger.error(message, *args, **kwargs)
        self.remote_log.ship_log(Logger.error.__name__, self.name, message)

    def info(self, message, *args, **kwargs):
        self.logger.info(message, *args, **kwargs)
        self.remote_log.ship_log(Logger.info.__name__, self.name, message)

    def critical(self, message, *args, **kwargs):
        self.logger.critical(message, *args, **kwargs)
        self.remote_log.ship_log(Logger.critical.__name__, self.name, message)


class LoggingProvider(Logger):
    def worker_setup(self, worker_ctx):
        self.log_name = worker_ctx.service_name

    def get_dependency(self, _):
        return self.logger


def log_exception(func):
    def wrapper(*args, **kwargs):
        try:
            return func(*args, **kwargs)
        except Exception as ex:
            Logger().error(f'Error during {func.__name__} {ex}', exc_info=True)
            raise
    return wrapper



import json
import os
import glob
from time import sleep
import requests
import yaml
from reliabilly.settings import Constants, Settings

EMPTY = ''
FILE_MODE = 'r'
NAME_KEY = 'name'
TESTS_KEY = 'tests'
DESCRIPTION_KEY = 'description'
VERB_KEY = 'verb'
GET = 'GET'
POST = 'POST'
PUT = 'PUT'
DELETE = 'DELETE'
WAIT = 'WAIT'
POST_ALL = 'POST_ALL'
ENDPOINT_KEY = 'endpoint'
STATUS_KEY = 'status'
TOTAL_KEY = 'total'
BODY_KEY = 'body'
TIME_KEY = 'time'
GREATER_KEY = 'greater'
TEST_CONFIG_PATH = '*.yml'
TARGET_KEY = 'target'
LOCAL_TARGET = 'local'
UAT_TARGET = 'uat'
PROD_TARGET = 'production'
LINE = '*' * 50


def get_headers():
    return {Constants.AUTHORIZATION_HEADER: Settings.AUTH_TOKEN, Constants.SERVICE_NAME_ENV: 'integration tests'}


def check_target_run(target, target_config):
    if not target_config:
        return True
    url_checkpoint = target.split('.')[0]
    if LOCAL_TARGET in url_checkpoint:
        return LOCAL_TARGET == target_config
    if UAT_TARGET in url_checkpoint:
        return UAT_TARGET == target_config
    if PROD_TARGET == target_config:
        return LOCAL_TARGET not in url_checkpoint and UAT_TARGET not in url_checkpoint
    return False


def run_service_integration_tests(target, service_path):
    for test_path in glob.glob(os.path.join(
            service_path, TEST_CONFIG_PATH)):
        with open(test_path, FILE_MODE) as test_file:
            test_config = yaml.load(test_file)
            print(LINE)
            target_config = test_config.get(TARGET_KEY, None)
            if check_target_run(target, target_config):
                print('Running {0}'.format(test_config[NAME_KEY]))
                test_cases = test_config[TESTS_KEY]
                run_test_cases(test_cases, target)


def run_test_cases(test_cases, target):
    for test in test_cases:
        print('Executing test: {0}'.format(test[DESCRIPTION_KEY]))
        verb = test[VERB_KEY]
        request_func = get_request_method(verb, test)
        target_endpoint = target + test.get(ENDPOINT_KEY, EMPTY)
        print(target_endpoint)
        if verb == WAIT:
            time = test.get(TIME_KEY, 30)  # Default to 30sec if not set
            print(f'Waiting {time}sec...')
            sleep(time)
            continue
        if verb == POST:
            payload = test.get(BODY_KEY, None)
            if test.get('file', None):
                payload = test['file']
            result = request_func(target_endpoint, data=payload, headers=get_headers())
        else:
            result = request_func(target_endpoint, headers=get_headers())
        print(f'Status: {result.status_code}, Expected: {test[STATUS_KEY]}')
        assert result.status_code == test[STATUS_KEY]
        body = test.get(BODY_KEY, None)
        total = test.get(TOTAL_KEY, None)
        if body and verb == GET:
            assert str(body) == result.text
        if total:
            greater = bool(test.get(GREATER_KEY, False))
            if greater:
                assert result.json()[TOTAL_KEY] > total
            else:
                assert result.json()[TOTAL_KEY] == total


def post_all_from_file(target_endpoint, data, **args):
    result = None
    with open('./data/{}'.format(data)) as file:
        items = json.load(file)
        for item in items:
            json_item = json.dumps(item)
            result = requests.post(target_endpoint, json_item, **args)
            if result.status_code != 201:
                return result
        return result


def get_request_method(verb, test):
    if verb == DELETE:
        return requests.delete
    if verb == POST:
        if test.get('file', None):
            return post_all_from_file
        return requests.post
    if verb == PUT:
        return requests.put
    return requests.get



# pylint: disable=invalid-name,no-self-use
import unittest
from unittest import skipUnless
from unittest.mock import MagicMock
from reliabilly.components.tools.data_monitoring_client import DataMonitoringClient
from reliabilly.settings import Settings, Constants


class DataMonitoringClientTests(unittest.TestCase):

    def test_generate_payload(self):
        payload = DataMonitoringClient.generate_payload(
            "99", "Updating", "on fire", "http://pipeline.com", "https://fresh.customgit.com")
        expected = {
            "eventType": "ReleaseEvent",
            "environment": "on fire",
            "build_status": "updating",
            "release_number": "99",
            "git_repo": "https://fresh.customgit.com",
            "build_pipeline": "http://pipeline.com",
        }
        self.assertEqual(payload, expected)

    def test_create_event(self):
        mock = MagicMock()
        client = DataMonitoringClient(mock, mock)
        result = client.create_event("1", "nice", "cool", "http://test.com", "https://fresh.customgit.com")
        self.assertTrue(result)

    def test_create_event_failure(self):
        mock = MagicMock()
        mock.create_event.return_value = False
        client = DataMonitoringClient(mock, mock)
        result = client.create_event("1", "nice", "cool", "http://test.com", "https://fresh.customgit.com")
        self.assertFalse(result)

    @skipUnless(Settings.RUN_SKIPPED, Constants.RUN_SKIPPED_MSG)
    def test_real_create_event(self):
        client = DataMonitoringClient('NEW_RELIC_API_KEY')
        client.create_event("123", "TESTING", "LOCAL", "http://hi.com", "http://test.net")



# pylint: disable=too-few-public-methods
from enum import Enum

from reliabilly.components.authorization import Authorization
from reliabilly.components.collection.collection_manager import CollectionManager
from reliabilly.components.controllers.collection_controller import CollectionController
from reliabilly.components.controllers.data_controller import DataController
from reliabilly.components.data.dynamo_client import DynamoDataClient
from reliabilly.components.data.mongo_client import MongoDataClient
from reliabilly.components.logger import Logger
from reliabilly.components.monitoring import MonitoringProvider
from reliabilly.components.services.newrelic import NewRelicQueryExecutor
from reliabilly.components.services.queuing import MessageQueue
from reliabilly.components.services.sumologic import SumologicQueryExecutor
from reliabilly.components.tools.secret_manager import SecretManager
from reliabilly.components.web.http_requestor import HttpRequestor


class Components:
    class DataClients(Enum):
        MONGODB = 1
        DYNAMODB = 2

    class LogClients(Enum):
        MAIN = 1

    class MonitorClients(Enum):
        STATSD = 1

    class SecretsClients(Enum):
        SECRETS = 1

    class ServiceClients(Enum):
        AUTH = 1
        NEWRELIC = 2
        OKTA = 3
        SUMOLOGIC = 4
        SQS = 5
        HTTP = 6
        COLLECT_MANAGER = 7

    class Controllers(Enum):
        DATA = 1
        COLLECT = 2


class Factory:  # pylint: disable=too-many-return-statements
    @staticmethod
    def create(component_type, **kwargs):
        if isinstance(component_type, Components.DataClients):
            return Factory._create_data_client(component_type, **kwargs)
        if isinstance(component_type, Components.ServiceClients):
            return Factory._create_service_client(component_type, **kwargs)
        if isinstance(component_type, Components.MonitorClients):
            return MonitoringProvider()
        if isinstance(component_type, Components.LogClients):
            return Logger()
        if isinstance(component_type, Components.SecretsClients):
            return SecretManager()
        if isinstance(component_type, Components.Controllers):
            return Factory._create_controller(component_type, **kwargs)
        return None

    @staticmethod
    def _create_controller(component_type, **kwargs):
        if component_type == Components.Controllers.DATA:
            return DataController(**kwargs)
        if component_type == Components.Controllers.COLLECT:
            return CollectionController(**kwargs)
        return None

    @staticmethod
    def _create_data_client(component_type, **kwargs):
        if component_type == Components.DataClients.MONGODB:
            return MongoDataClient(**kwargs)
        if component_type == Components.DataClients.DYNAMODB:
            return DynamoDataClient(**kwargs)
        return None

    @staticmethod
    def _create_service_client(component_type, **kwargs):
        if component_type == Components.ServiceClients.SQS:
            return MessageQueue(**kwargs)
        if component_type == Components.ServiceClients.NEWRELIC:
            return NewRelicQueryExecutor(**kwargs)
        if component_type == Components.ServiceClients.SUMOLOGIC:
            return SumologicQueryExecutor(**kwargs)
        if component_type == Components.ServiceClients.HTTP:
            return HttpRequestor(**kwargs)
        if component_type == Components.ServiceClients.AUTH:
            return Authorization()
        if component_type == Components.ServiceClients.COLLECT_MANAGER:
            return CollectionManager(**kwargs)
        return None



from unittest import TestCase
from examples.data_mongo.controllers.data_mongo_controller import DataMongoController
from examples.data_mongo.service import RUNNER


class DataMongoTests(TestCase):
    def test_all(self):
        controller = DataMongoController()
        self.assertIsNotNone(controller)

    def test_service(self):
        self.assertIsNotNone(RUNNER)
